#!/usr/bin/python

# Copyright (C) 2003. Pedro Ferreira
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

import BoostBuild


###############################################################################
#
# test_folder_with_dot_in_name()
# ------------------------------
#
###############################################################################

def test_folder_with_dot_in_name(t):
    """
      Regression test: the 'tag' feature did not work in directories that had a
    dot in their name.

    """
    t.write("version-1.32.0/jamroot.jam", """\
project test : requirements <tag>@$(__name__).tag ;

rule tag ( name : type ? : property-set )
{
   # Do nothing, just make sure the rule is invoked OK.
   ECHO The tag rule has been invoked. ;
}
exe a : a.cpp ;
""")
    t.write("version-1.32.0/a.cpp", "int main() {}\n")

    t.run_build_system(subdir="version-1.32.0")
    t.expect_addition("version-1.32.0/bin/$toolset/debug*/a.exe")
    t.expect_output_lines("The tag rule has been invoked.")


###############################################################################
#
# test_tag_property()
# -------------------
#
###############################################################################

def test_tag_property(t):
    """Basic tag property test."""

    t.write("jamroot.jam", """\
import virtual-target ;

rule tag ( name : type ? : property-set )
{
    local tags ;
    switch [ $(property-set).get <variant> ]
    {
        case debug   : tags += d ;
        case release : tags += r ;
    }
    switch [ $(property-set).get <link> ]
    {
        case shared : tags += s ;
        case static : tags += t ;
    }
    if $(tags)
    {
        return [ virtual-target.add-prefix-and-suffix $(name)_$(tags:J="")
            : $(type) : $(property-set) ] ;
    }
}

# Test both fully-qualified and local name of the rule
exe a : a.cpp : <tag>@$(__name__).tag ;
lib b : a.cpp : <tag>@tag ;
stage c : a ;
""")

    t.write("a.cpp", """\
int main() {}
#ifdef _MSC_VER
__declspec (dllexport) void x () {}
#endif
""")

    file_list = (
        BoostBuild.List("bin/$toolset/debug*/a_ds.exe") +
        BoostBuild.List("bin/$toolset/debug*/b_ds.dll") +
        BoostBuild.List("c/a_ds.exe") +
        BoostBuild.List("bin/$toolset/release*/a_rs.exe") +
        BoostBuild.List("bin/$toolset/release*/b_rs.dll") +
        BoostBuild.List("c/a_rs.exe") +
        BoostBuild.List("bin/$toolset/debug/link-static*/a_dt.exe") +
        BoostBuild.List("bin/$toolset/debug/link-static*/b_dt.lib") +
        BoostBuild.List("c/a_dt.exe") +
        BoostBuild.List("bin/$toolset/release/link-static*/a_rt.exe") +
        BoostBuild.List("bin/$toolset/release/link-static*/b_rt.lib") +
        BoostBuild.List("c/a_rt.exe"))

    variants = ["debug", "release", "link=static,shared"]

    t.run_build_system(variants)
    t.expect_addition(file_list)

    t.run_build_system(variants + ["clean"])
    t.expect_removal(file_list)


###############################################################################
#
# main()
# ------
#
###############################################################################

t = BoostBuild.Tester(use_test_config=False)

test_tag_property(t)
test_folder_with_dot_in_name(t)

t.cleanup()



#!/usr/bin/python

# Copyright (C) 2013 Steven Watanabe
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

import BoostBuild
import MockToolset

t = BoostBuild.Tester(arguments=['toolset=mock', '--ignore-site-config', '--user-config='], pass_toolset=0)

MockToolset.create(t)

# Build from source
t.write("zlib/zlib.h", 'zlib')
t.write("zlib/deflate.c", 'deflate')

t.write("Jamroot.jam", """
path-constant here : . ;
using zlib : : <source>$(here)/zlib ;
alias zlib : /zlib//zlib : : <link>static <link>shared ;
""")

MockToolset.set_expected(t, '''
source_file('deflate.c', 'deflate')
action('-c -x c -I./zlib -o $deflate.o $deflate.c')
action('--dll $deflate.o -o $deflate.so')
action('--archive $deflate.o -o $deflate.a')
''')

t.run_build_system()
t.expect_addition('bin/standalone/zlib/mock/debug/z.dll')
t.expect_addition('bin/standalone/zlib/mock/debug/link-static/z.lib')

t.rm('zlib')

# Generic definitions that aren't configuration specific
common_stuff = '''
source_file('test.cpp', 'test.cpp')
source_file('main.cpp', 'int main() {}')
source_file('zlib.h.cpp', '#include <zlib.h>\\n')
action('-c -x c++ $main.cpp -o $main.o')
'''
t.write('test.cpp', 'test.cpp')

# Default initialization - static library
t.rm('bin')
t.write("Jamroot.jam", """
path-constant here : . ;
using zlib ;
exe test : test.cpp /zlib//zlib : : <link>static <link>shared ;
""")

MockToolset.set_expected(t, common_stuff + '''
action('$main.o --static-lib=z -o $config.exe')
action('-c -x c++ $zlib.h.cpp -o $zlib.h.o')
action('-c -x c++ $test.cpp -o $test.o')
action('$test.o --static-lib=z -o $test')
''')
t.run_build_system()
t.expect_addition('bin/mock/debug/test.exe')
t.expect_addition('bin/mock/debug/link-static/test.exe')

# Default initialization - shared library
t.rm('bin')
t.write("Jamroot.jam", """
path-constant here : . ;
using zlib ;
exe test : test.cpp /zlib//zlib : : <link>static <link>shared ;
""")

MockToolset.set_expected(t, common_stuff + '''
action('$main.o --shared-lib=z -o $config.exe')
action('-c -x c++ $zlib.h.cpp -o $zlib.h.o')
action('-c -x c++ $test.cpp -o $test.o')
action('$test.o --shared-lib=z -o $test')
''')
t.run_build_system()
t.expect_addition('bin/mock/debug/test.exe')
t.expect_addition('bin/mock/debug/link-static/test.exe')

# Initialization in explicit location - static library
t.rm('bin')
t.write("Jamroot.jam", """
path-constant here : . ;
using zlib : : <name>myzlib <include>$(here)/zlib <search>$(here)/zlib ;
exe test : test.cpp /zlib//zlib : : <link>static <link>shared ;
""")

t.write('zlib/zlib.h', 'zlib')

MockToolset.set_expected(t, common_stuff + '''
action('$main.o -L./zlib --static-lib=myzlib -o $config.exe')
action('-c -x c++ $test.cpp -I./zlib -o $test.o')
action('$test.o -L./zlib --static-lib=myzlib -o $test')
''')
t.run_build_system()
t.expect_addition('bin/mock/debug/test.exe')
t.expect_addition('bin/mock/debug/link-static/test.exe')

# Initialization in explicit location - shared library
t.rm('bin')
t.write("Jamroot.jam", """
path-constant here : . ;
using zlib : : <name>myzlib <include>$(here)/zlib <search>$(here)/zlib ;
exe test : test.cpp /zlib//zlib : : <link>static <link>shared ;
""")

MockToolset.set_expected(t, common_stuff + '''
action('$main.o -L./zlib --shared-lib=myzlib -o $config.exe')
action('-c -x c++ $test.cpp -I./zlib -o $test.o')
action('$test.o -L./zlib --shared-lib=myzlib -o $test')
''')
t.run_build_system()
t.expect_addition('bin/mock/debug/test.exe')
t.expect_addition('bin/mock/debug/link-static/test.exe')

t.cleanup()



#!/usr/bin/python

# Copyright 2003 Dave Abrahams
# Copyright 2002, 2003, 2004, 2006 Vladimir Prus
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

import BoostBuild

t = BoostBuild.Tester(use_test_config=False)


# Test that usage requirements on main targets work (and are propagated all the
# way up, and not only to direct dependants).
t.write("jamroot.jam", "")

# Note: 'lib cc ..', not 'lib c'. If using 'lib c: ...' the HP-CXX linker will
# confuse it with the system C runtime.
t.write("jamfile.jam", """\
lib b : b.cpp : <link>shared:<define>SHARED_B : :
    <define>FOO <link>shared:<define>SHARED_B ;
lib cc : c.cpp b ;
exe a : a.cpp cc ;
""")

t.write("b.cpp", """\
void
#if defined(_WIN32) && defined(SHARED_B)
__declspec(dllexport)
#endif
foo() {}
""")

t.write("c.cpp", """\
void
#if defined(_WIN32) && defined(SHARED_B)
__declspec(dllexport)
#endif
create_lib_please() {}
""")

t.write("a.cpp", """\
#ifdef FOO
void
# if defined(_WIN32) && defined(SHARED_B)
__declspec(dllexport)
# endif
foo() {}
#endif
int main() { foo(); }
""")

t.run_build_system()
t.run_build_system(["--clean"])


# Test that use requirements on main target work, when they are referred using
# 'dependency' features.

t.write("jamfile.jam", """\
lib b : b.cpp : <link>shared:<define>SHARED_B : : <define>FOO
    <link>shared:<define>SHARED_B ;
exe a : a.cpp : <use>b ;
""")

t.write("b.cpp", """\
void
#if defined(_WIN32) && defined(SHARED_B)
__declspec(dllexport)
#endif
foo() {}
""")

t.write("a.cpp", """\
#ifdef FOO
int main() {}
#endif
""")

t.run_build_system()
t.run_build_system(["--clean"])


# Test that usage requirements on a project work.
t.write("jamfile.jam", "exe a : a.cpp lib//b ;")

t.write("lib/jamfile.jam", """\
project
   : requirements <link>shared:<define>SHARED_B
   : usage-requirements <define>FOO <link>shared:<define>SHARED_B ;
lib b : b.cpp ;
""")

t.write("lib/b.cpp", """\
void
#if defined(_WIN32) && defined(SHARED_B)
__declspec(dllexport)
#endif
foo() {}
""")

t.run_build_system()


# Test that use requirements are inherited correctly.
t.write("jamfile.jam", "exe a : a.cpp lib/1//b ;")

t.write("a.cpp", """\
#if defined(FOO) && defined(ZOO)
void foo() {}
#endif
int main() { foo(); }
""")

t.write("lib/jamfile.jam", """\
project : requirements : usage-requirements <define>FOO ;
""")

t.write("lib/1/jamfile.jam", """\
project
   : requirements <link>shared:<define>SHARED_B
   : usage-requirements <define>ZOO <link>shared:<define>SHARED_B ;
lib b : b.cpp ;
""")

t.write("lib/1/b.cpp", """\
void
#if defined(_WIN32) && defined(SHARED_B)
__declspec(dllexport)
#endif
foo() {}
""")

t.run_build_system()
t.run_build_system(["--clean"])


# Test that we correctly handle dependency features in usage requirements on
# target.
t.write("jamfile.jam", """\
lib b : b.cpp : <link>shared:<define>SHARED_B : : <define>FOO
    <link>shared:<define>SHARED_B ;

# Here's the test: we should correctly handle dependency feature and get usage
# requirements from 'b'.
lib cc : c.cpp : <link>shared:<define>SHARED_C : : <library>b ;

# This will build only if <define>FOO was propagated from 'c'.
exe a : a.cpp cc ;
""")

t.write("a.cpp", """\
#ifdef FOO
void
# if defined(_WIN32) && defined(SHARED_B)
__declspec(dllexport)
# endif
foo();
#endif

int main() { foo(); }
""")

t.write("c.cpp", """\
int
#if defined(_WIN32) && defined(SHARED_C)
__declspec(dllexport)
#endif
must_export_something;
""")

t.run_build_system()
t.run_build_system(["--clean"])


# Test correct handling of dependency features in project requirements.
t.write("jamfile.jam", "exe a : a.cpp lib1//cc ;")

t.write("lib1/jamfile.jam", """\
project
    : requirements <link>shared:<define>SHARED_C
    : usage-requirements <library>../lib2//b <link>shared:<define>SHARED_C ;
lib cc : c.cpp ;
""")

t.write("lib1/c.cpp", """\
int
#if defined(_WIN32) && defined(SHARED_C)
__declspec(dllexport)
#endif
must_export_something;
""")

t.write("lib2/jamfile.jam", """\
lib b : b.cpp : <link>shared:<define>SHARED_B : : <define>FOO
    <link>shared:<define>SHARED_B ;
""")

t.copy("b.cpp", "lib2/b.cpp")

t.run_build_system()


# Test that targets listed in dependency features in usage requirements are
# built with the correct properties.
t.rm(".")

t.write("jamroot.jam", "")
t.write("jamfile.jam", """\
lib main : main.cpp : <use>libs//lib1 : : <library>libs//lib1 ;
exe hello : hello.cpp main : ;
""")

t.write("main.cpp", """\
void
#if defined(_WIN32) && defined(SHARED_LIB1)
__declspec(dllimport)
#endif
foo();

int main() { foo(); }
""")

t.write("hello.cpp", "\n")
t.write("libs/a.cpp", """\
void
#if defined(_WIN32) && defined(SHARED_LIB1)
__declspec(dllexport)
#endif
foo() {}
""")


# This library should be built with the same properties as 'main'. This is a
# regression test for a bug when they were generated with empty properties, and
# there were ambiguities between variants.
t.write("libs/jamfile.jam", """\
lib lib1 : a_d.cpp : <variant>debug <link>shared:<define>SHARED_LIB1 : :
    <link>shared:<define>SHARED_LIB1 ;
lib lib1 : a.cpp : <variant>release <link>shared:<define>SHARED_LIB1 : :
    <link>shared:<define>SHARED_LIB1 ;
""")

t.write("libs/a_d.cpp", """\
void
#if defined(_WIN32) && defined(SHARED_LIB1)
__declspec(dllexport)
#endif
foo() {}
""")

t.run_build_system(["link=static"])
t.expect_addition("libs/bin/$toolset/debug/link-static*/a_d.obj")


# Test that indirect conditionals are respected in usage requirements.
t.rm(".")

t.write("jamroot.jam", """\
rule has-foo ( properties * ) { return <define>HAS_FOO ; }
exe a : a.cpp b ;
lib b : b.cpp : <link>static : : <conditional>@has-foo ;
""")

t.write("a.cpp", """\
#ifdef HAS_FOO
void foo();
int main() { foo(); }
#endif
""")

t.write("b.cpp", """\
void
#if defined(_WIN32) && defined(SHARED_B)
__declspec(dllexport)
#endif
foo() {}
""")

t.run_build_system()
t.expect_addition("bin/$toolset/debug*/a.exe")

t.cleanup()



#!/usr/bin/python

# Copyright 2003 Vladimir Prus
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

# Test that C files are compiled by a C compiler.

import BoostBuild

t = BoostBuild.Tester(use_test_config=False)

t.write("jamroot.jam", """
project ;
exe hello : hello.cpp a.c ;
""")

t.write("hello.cpp", """
extern "C" int foo();
int main() { return foo(); }
""")

t.write("a.c", """
// This will not compile unless in C mode.
int foo()
{
    int new = 0;
    new = (new+1)*7;
    return new;
}
""")

t.run_build_system()
t.expect_addition("bin/$toolset/debug*/hello.exe")

t.cleanup()



# Niklaus Giger, 2005-03-15
# Testing whether we may run a test in absolute directories. There are no tests
# for temporary directories as this is implictly tested in a lot of other cases.

# TODO: Move to a separate testing-system test group.
# TODO: Make the test not display any output on success.
# TODO: Make sure implemented path handling is correct under Windows, Cygwin &
#       Unix/Linux.

import BoostBuild
import os
import tempfile

t = BoostBuild.Tester(["-ffile.jam"], workdir=os.getcwd(), pass_d0=False,
    pass_toolset=False)

t.write("file.jam", "EXIT [ PWD ] : 0 ;")

t.run_build_system()
t.expect_output_lines("*%s*" % tempfile.gettempdir(), False)
t.expect_output_lines("*build/v2/test*")

t.run_build_system(status=1, subdir="/must/fail/with/absolute/path",
    stderr=None)

t.cleanup()



#!/usr/bin/python

# Copyright 2004 Vladimir Prus
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

# Test that the <library> property has no effect on "obj" targets. Previously,
# it affected all targets, so
#
#    project : requirements <library>foo ;
#    exe a : a.cpp helper ;
#    obj helper : helper.cpp : <optimization>off ;
#
# caused 'foo' to be built with and without optimization.

import BoostBuild

t = BoostBuild.Tester(use_test_config=False)

t.write("jamroot.jam", """
project : requirements <library>lib//x ;
exe a : a.cpp foo ;
obj foo : foo.cpp : <variant>release ;
""")

t.write("a.cpp", """
void aux();
int main() { aux(); }
""")

t.write("foo.cpp", """
void gee();
void aux() { gee(); }
""")

t.write("lib/x.cpp", """
void
#if defined(_WIN32)
__declspec(dllexport)
#endif
gee() {}
""")

t.write("lib/jamfile.jam", """
lib x : x.cpp ;
""")

t.write("lib/jamroot.jam", """
""")


t.run_build_system()
t.expect_addition("bin/$toolset/debug*/a.exe")
t.expect_nothing("lib/bin/$toolset/release/x.obj")

t.cleanup()



#!/usr/bin/python

# Copyright 2003 Vladimir Prus
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

import BoostBuild

t = BoostBuild.Tester(pass_toolset=0)

t.write("code", """\
module a
{
    rule r1 ( )
    {
        ECHO R1 ;
    }

    local rule l1 ( )
    {
        ECHO A.L1 ;
    }
}
module a2
{
    rule r2 ( )
    {
        ECHO R2 ;
    }
}
IMPORT a2 : r2 : : a2.r2 ;

rule a.l1 ( )
{
    ECHO L1 ;
}

module b
{
    IMPORT_MODULE a : b ;
    rule test
    {
        # Call rule visible via IMPORT_MODULE
        a.r1 ;
        # Call rule in global scope
        a2.r2 ;
        # Call rule in global scope.  Doesn't find local rule
        a.l1 ;
        # Make l1 visible
        EXPORT a : l1 ;
        a.l1 ;
    }
}

IMPORT b : test : : test ;
test ;

module c
{
    rule test
    {
        ECHO CTEST ;
    }
}

IMPORT_MODULE c : ;
c.test ;

actions do-nothing { }
do-nothing all ;
""")

t.run_build_system(["-fcode"], stdout="""\
R1
R2
L1
A.L1
CTEST
""")

t.cleanup()



#  Copyright (C) Christopher Currie 2003. Permission to copy, use,
#  modify, sell and distribute this software is granted provided this
#  copyright notice appears in all copies. This software is provided
# "as is" without express or implied warranty, and with no claim as to
#  its suitability for any purpose.

#  Please see http://article.gmane.org/gmane.comp.lib.boost.build/3389/
#  for explanation why it's a separate toolset.

import common, gcc, builtin
from b2.build import feature, toolset, type, action, generators
from b2.util.utility import *

toolset.register ('darwin')

toolset.inherit_generators ('darwin', [], 'gcc')
toolset.inherit_flags ('darwin', 'gcc')
toolset.inherit_rules ('darwin', 'gcc')

def init (version = None, command = None, options = None):
    options = to_seq (options)

    condition = common.check_init_parameters ('darwin', None, ('version', version))

    command = common.get_invocation_command ('darwin', 'g++', command)

    common.handle_options ('darwin', condition, command, options)

    gcc.init_link_flags ('darwin', 'darwin', condition)

# Darwin has a different shared library suffix
type.set_generated_target_suffix ('SHARED_LIB', ['<toolset>darwin'], 'dylib')

# we need to be able to tell the type of .dylib files
type.register_suffixes ('dylib', 'SHARED_LIB')

feature.feature ('framework', [], ['free'])

toolset.flags ('darwin.compile', 'OPTIONS', '<link>shared', ['-dynamic'])
toolset.flags ('darwin.compile', 'OPTIONS', None, ['-Wno-long-double', '-no-cpp-precomp'])
toolset.flags ('darwin.compile.c++', 'OPTIONS', None, ['-fcoalesce-templates'])

toolset.flags ('darwin.link', 'FRAMEWORK', '<framework>')

# This is flag is useful for debugging the link step
# uncomment to see what libtool is doing under the hood
# toolset.flags ('darwin.link.dll', 'OPTIONS', None, '[-Wl,-v'])

action.register ('darwin.compile.cpp', None, ['$(CONFIG_COMMAND) $(ST_OPTIONS) -L"$(LINKPATH)" -o "$(<)" "$(>)" "$(LIBRARIES)" -l$(FINDLIBS-SA) -l$(FINDLIBS-ST) -framework$(_)$(FRAMEWORK) $(OPTIONS)'])

# TODO: how to set 'bind LIBRARIES'?
action.register ('darwin.link.dll', None, ['$(CONFIG_COMMAND) -dynamiclib -L"$(LINKPATH)" -o "$(<)" "$(>)" "$(LIBRARIES)" -l$(FINDLIBS-SA) -l$(FINDLIBS-ST) -framework$(_)$(FRAMEWORK) $(OPTIONS)'])

def darwin_archive (manager, targets, sources, properties):
    pass

action.register ('darwin.archive', darwin_archive, ['ar -c -r -s $(ARFLAGS) "$(<:T)" "$(>:T)"'])



#!/usr/bin/python

#  Copyright (C) Vladimir Prus 2006.
#  Distributed under the Boost Software License, Version 1.0. (See
#  accompanying file LICENSE_1_0.txt or copy at
#  http://www.boost.org/LICENSE_1_0.txt)

import BoostBuild

t = BoostBuild.Tester(use_test_config=False)

t.write("a.cpp", "int main() {}\n")
t.write("jamroot.jam", "exe a : a.cpp sub1//sub1 sub2//sub2 sub3//sub3 ;")
t.write("sub1/jamfile.jam", """\
lib sub1 : sub1.cpp sub1_2 ../sub2//sub2 ;
lib sub1_2 : sub1_2.cpp ;
""")

t.write("sub1/sub1.cpp", """\
#ifdef _WIN32
__declspec(dllexport)
#endif
void sub1() {}
""")

t.write("sub1/sub1_2.cpp", """\
#ifdef _WIN32
__declspec(dllexport)
#endif
void sub1() {}
""")

t.write("sub2/jamfile.jam", "lib sub2 : sub2.cpp ;")
t.write("sub2/sub2.cpp", """\
#ifdef _WIN32
__declspec(dllexport)
#endif
void sub2() {}
""")

t.write("sub3/jamroot.jam", "lib sub3 : sub3.cpp ;")
t.write("sub3/sub3.cpp", """\
#ifdef _WIN32
__declspec(dllexport)
#endif
void sub3() {}
""")

# 'clean' should not remove files under separate jamroot.jam.
t.run_build_system()
t.run_build_system(["--clean"])
t.expect_removal("bin/$toolset/debug*/a.obj")
t.expect_removal("sub1/bin/$toolset/debug*/sub1.obj")
t.expect_removal("sub1/bin/$toolset/debug*/sub1_2.obj")
t.expect_removal("sub2/bin/$toolset/debug*/sub2.obj")
t.expect_nothing("sub3/bin/$toolset/debug*/sub3.obj")

# 'clean-all' removes everything it can reach.
t.run_build_system()
t.run_build_system(["--clean-all"])
t.expect_removal("bin/$toolset/debug*/a.obj")
t.expect_removal("sub1/bin/$toolset/debug*/sub1.obj")
t.expect_removal("sub1/bin/$toolset/debug*/sub1_2.obj")
t.expect_removal("sub2/bin/$toolset/debug*/sub2.obj")
t.expect_nothing("sub3/bin/$toolset/debug*/sub3.obj")

# 'clean' together with project target removes only under that project.
t.run_build_system()
t.run_build_system(["sub1", "--clean"])
t.expect_nothing("bin/$toolset/debug*/a.obj")
t.expect_removal("sub1/bin/$toolset/debug*/sub1.obj")
t.expect_removal("sub1/bin/$toolset/debug*/sub1_2.obj")
t.expect_nothing("sub2/bin/$toolset/debug*/sub2.obj")
t.expect_nothing("sub3/bin/$toolset/debug*/sub3.obj")

# 'clean-all' removes everything.
t.run_build_system()
t.run_build_system(["sub1", "--clean-all"])
t.expect_nothing("bin/$toolset/debug*/a.obj")
t.expect_removal("sub1/bin/$toolset/debug*/sub1.obj")
t.expect_removal("sub1/bin/$toolset/debug*/sub1_2.obj")
t.expect_removal("sub2/bin/$toolset/debug*/sub2.obj")
t.expect_nothing("sub3/bin/$toolset/debug*/sub3.obj")

# If main target is explicitly named, we should not remove files from other
# targets.
t.run_build_system()
t.run_build_system(["sub1//sub1", "--clean"])
t.expect_removal("sub1/bin/$toolset/debug*/sub1.obj")
t.expect_nothing("sub1/bin/$toolset/debug*/sub1_2.obj")
t.expect_nothing("sub2/bin/$toolset/debug*/sub2.obj")
t.expect_nothing("sub3/bin/$toolset/debug*/sub3.obj")

# Regression test: sources of the 'cast' rule were mistakenly deleted.
t.rm(".")
t.write("jamroot.jam", """\
import cast ;
cast a cpp : a.h ;
""")
t.write("a.h", "")
t.run_build_system(["--clean"])
t.expect_nothing("a.h")

t.cleanup()



# Status: mostly ported. Missing is --out-xml support, 'configure' integration
# and some FIXME.
# Base revision: 64351

# Copyright 2003, 2005 Dave Abrahams
# Copyright 2006 Rene Rivera
# Copyright 2003, 2004, 2005, 2006, 2007 Vladimir Prus
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)
import os
import sys
import re

import bjam

# set this early on since some of the following modules
# require looking at the sys.argv
sys.argv = bjam.variable("ARGV")


from b2.build.engine import Engine
from b2.manager import Manager
from b2.util.path import glob
from b2.build import feature, property_set
import b2.build.virtual_target
from b2.build.targets import ProjectTarget
import b2.build.build_request
from b2.build.errors import ExceptionWithUserContext
import b2.tools.common
from b2.build.toolset import using

import b2.build.virtual_target as virtual_target
import b2.build.build_request as build_request

import b2.util.regex

from b2.manager import get_manager
from b2.util import cached
from b2.util import option

################################################################################
#
# Module global data.
#
################################################################################

# Flag indicating we should display additional debugging information related to
# locating and loading Boost Build configuration files.
debug_config = False

# The cleaning is tricky. Say, if user says 'bjam --clean foo' where 'foo' is a
# directory, then we want to clean targets which are in 'foo' as well as those
# in any children Jamfiles under foo but not in any unrelated Jamfiles. To
# achieve this we collect a list of projects under which cleaning is allowed.
project_targets = []

# Virtual targets obtained when building main targets references on the command
# line. When running 'bjam --clean main_target' we want to clean only files
# belonging to that main target so we need to record which targets are produced
# for it.
results_of_main_targets = []

# Was an XML dump requested?
out_xml = False

# Default toolset & version to be used in case no other toolset has been used
# explicitly by either the loaded configuration files, the loaded project build
# scripts or an explicit toolset request on the command line. If not specified,
# an arbitrary default will be used based on the current host OS. This value,
# while not strictly necessary, has been added to allow testing Boost-Build's
# default toolset usage functionality.
default_toolset = None
default_toolset_version = None

################################################################################
#
# Public rules.
#
################################################################################

# Returns the property set with the free features from the currently processed
# build request.
#
def command_line_free_features():
    return command_line_free_features

# Sets the default toolset & version to be used in case no other toolset has
# been used explicitly by either the loaded configuration files, the loaded
# project build scripts or an explicit toolset request on the command line. For
# more detailed information see the comment related to used global variables.
#
def set_default_toolset(toolset, version=None):
    default_toolset = toolset
    default_toolset_version = version


pre_build_hook = []

def add_pre_build_hook(callable):
    pre_build_hook.append(callable)

post_build_hook = None

def set_post_build_hook(callable):
    post_build_hook = callable

################################################################################
#
# Local rules.
#
################################################################################

# Returns actual Jam targets to be used for executing a clean request.
#
def actual_clean_targets(targets):

    # Construct a list of projects explicitly detected as targets on this build
    # system run. These are the projects under which cleaning is allowed.
    for t in targets:
        if isinstance(t, b2.build.targets.ProjectTarget):
            project_targets.append(t.project_module())

    # Construct a list of targets explicitly detected on this build system run
    # as a result of building main targets.
    targets_to_clean = set()
    for t in results_of_main_targets:
        # Do not include roots or sources.
        targets_to_clean.update(virtual_target.traverse(t))

    to_clean = []
    for t in get_manager().virtual_targets().all_targets():

        # Remove only derived targets.
        if t.action():
            p = t.project()
            if t in targets_to_clean or should_clean_project(p.project_module()):
                to_clean.append(t)

    return [t.actualize() for t in to_clean]

_target_id_split = re.compile("(.*)//(.*)")

# Given a target id, try to find and return the corresponding target. This is
# only invoked when there is no Jamfile in ".". This code somewhat duplicates
# code in project-target.find but we can not reuse that code without a
# project-targets instance.
#
def find_target(target_id):

    projects = get_manager().projects()
    m = _target_id_split.match(target_id)
    if m:
        pm = projects.find(m.group(1), ".")
    else:
        pm = projects.find(target_id, ".")

    if pm:
        result = projects.target(pm)

    if m:
        result = result.find(m.group(2))

    return result

def initialize_config_module(module_name, location=None):

    get_manager().projects().initialize(module_name, location)

# Helper rule used to load configuration files. Loads the first configuration
# file with the given 'filename' at 'path' into module with name 'module-name'.
# Not finding the requested file may or may not be treated as an error depending
# on the must-find parameter. Returns a normalized path to the loaded
# configuration file or nothing if no file was loaded.
#
def load_config(module_name, filename, paths, must_find=False):

    if debug_config:
        print "notice: Searching  '%s' for '%s' configuration file '%s." \
              % (paths, module_name, filename)

    where = None
    for path in paths:
        t = os.path.join(path, filename)
        if os.path.exists(t):
            where = t
            break

    if where:
        where = os.path.realpath(where)

        if debug_config:
            print "notice: Loading '%s' configuration file '%s' from '%s'." \
                  % (module_name, filename, where)

        # Set source location so that path-constant in config files
        # with relative paths work. This is of most importance
        # for project-config.jam, but may be used in other
        # config files as well.
        attributes = get_manager().projects().attributes(module_name) ;
        attributes.set('source-location', os.path.dirname(where), True)
        get_manager().projects().load_standalone(module_name, where)

    else:
        msg = "Configuration file '%s' not found in '%s'." % (filename, path)
        if must_find:
            get_manager().errors()(msg)

        elif debug_config:
            print msg

    return where

# Loads all the configuration files used by Boost Build in the following order:
#
#   -- test-config --
#   Loaded only if specified on the command-line using the --test-config
# command-line parameter. It is ok for this file not to exist even if
# specified. If this configuration file is loaded, regular site and user
# configuration files will not be. If a relative path is specified, file is
# searched for in the current folder.
#
#   -- site-config --
#   Always named site-config.jam. Will only be found if located on the system
# root path (Windows), /etc (non-Windows), user's home folder or the Boost
# Build path, in that order. Not loaded in case the test-config configuration
# file is loaded or the --ignore-site-config command-line option is specified.
#
#   -- user-config --
#   Named user-config.jam by default or may be named explicitly using the
# --user-config command-line option or the BOOST_BUILD_USER_CONFIG environment
# variable. If named explicitly the file is looked for from the current working
# directory and if the default one is used then it is searched for in the
# user's home directory and the Boost Build path, in that order. Not loaded in
# case either the test-config configuration file is loaded or an empty file
# name is explicitly specified. If the file name has been given explicitly then
# the file must exist.
#
# Test configurations have been added primarily for use by Boost Build's
# internal unit testing system but may be used freely in other places as well.
#
def load_configuration_files():

    # Flag indicating that site configuration should not be loaded.
    ignore_site_config = "--ignore-site-config" in sys.argv

    initialize_config_module("test-config")
    test_config = None
    for a in sys.argv:
        m = re.match("--test-config=(.*)$", a)
        if m:
            test_config = b2.util.unquote(m.group(1))
            break

    if test_config:
        where = load_config("test-config", os.path.basename(test_config), [os.path.dirname(test_config)])
        if where:
            if debug_config:
                print "notice: Regular site and user configuration files will"
                print "notice: be ignored due to the test configuration being loaded."

    user_path = [os.path.expanduser("~")] + bjam.variable("BOOST_BUILD_PATH")
    site_path = ["/etc"] + user_path
    if os.name in ["nt"]:
        site_path = [os.getenv("SystemRoot")] + user_path

    if debug_config and not test_config and ignore_site_config:
        print "notice: Site configuration files will be ignored due to the"
        print "notice: --ignore-site-config command-line option."

    initialize_config_module("site-config")
    if not test_config and not ignore_site_config:
        load_config('site-config', 'site-config.jam', site_path)

    initialize_config_module('user-config')
    if not test_config:

        # Here, user_config has value of None if nothing is explicitly
        # specified, and value of '' if user explicitly does not want
        # to load any user config.
        user_config = None
        for a in sys.argv:
            m = re.match("--user-config=(.*)$", a)
            if m:
                user_config = m.group(1)
                break

        if user_config is None:
            user_config = os.getenv("BOOST_BUILD_USER_CONFIG")

        # Special handling for the case when the OS does not strip the quotes
        # around the file name, as is the case when using Cygwin bash.
        user_config = b2.util.unquote(user_config)
        explicitly_requested = user_config

        if user_config is None:
            user_config = "user-config.jam"

        if user_config:
            if explicitly_requested:

                user_config = os.path.abspath(user_config)

                if debug_config:
                    print "notice: Loading explicitly specified user configuration file:"
                    print "    " + user_config

                    load_config('user-config', os.path.basename(user_config), [os.path.dirname(user_config)], True)
            else:
                load_config('user-config', os.path.basename(user_config), user_path)
        else:
            if debug_config:
                print "notice: User configuration file loading explicitly disabled."

    # We look for project-config.jam from "." upward. I am not sure this is
    # 100% right decision, we might as well check for it only alongside the
    # Jamroot file. However:
    # - We need to load project-config.jam before Jamroot
    # - We probably need to load project-config.jam even if there is no Jamroot
    #   - e.g. to implement automake-style out-of-tree builds.
    if os.path.exists("project-config.jam"):
        file = ["project-config.jam"]
    else:
        file = b2.util.path.glob_in_parents(".", ["project-config.jam"])

    if file:
        initialize_config_module('project-config', os.path.dirname(file[0]))
        load_config('project-config', "project-config.jam", [os.path.dirname(file[0])], True)

    get_manager().projects().end_load()


# Autoconfigure toolsets based on any instances of --toolset=xx,yy,...zz or
# toolset=xx,yy,...zz in the command line. May return additional properties to
# be processed as if they had been specified by the user.
#
def process_explicit_toolset_requests():

    extra_properties = []

    option_toolsets = [e for option in b2.util.regex.transform(sys.argv, "^--toolset=(.*)$")
                       for e in option.split(',')]
    feature_toolsets = [e for option in b2.util.regex.transform(sys.argv, "^toolset=(.*)$")
                       for e in option.split(',')]

    for t in option_toolsets + feature_toolsets:

        # Parse toolset-version/properties.
        (toolset_version, toolset, version) = re.match("(([^-/]+)-?([^/]+)?)/?.*", t).groups()

        if debug_config:
            print "notice: [cmdline-cfg] Detected command-line request for '%s': toolset= %s version=%s" \
            % (toolset_version, toolset, version)

        # If the toolset is not known, configure it now.
        known = False
        if toolset in feature.values("toolset"):
            known = True

        if known and version and not feature.is_subvalue("toolset", toolset, "version", version):
            known = False
        # TODO: we should do 'using $(toolset)' in case no version has been
        # specified and there are no versions defined for the given toolset to
        # allow the toolset to configure its default version. For this we need
        # to know how to detect whether a given toolset has any versions
        # defined. An alternative would be to do this whenever version is not
        # specified but that would require that toolsets correctly handle the
        # case when their default version is configured multiple times which
        # should be checked for all existing toolsets first.

        if not known:

            if debug_config:
                print "notice: [cmdline-cfg] toolset '%s' not previously configured; attempting to auto-configure now" % toolset_version
            if version is not None:
               using(toolset, version)
            else:
               using(toolset)

        else:

            if debug_config:

                print "notice: [cmdline-cfg] toolset '%s' already configured" % toolset_version

        # Make sure we get an appropriate property into the build request in
        # case toolset has been specified using the "--toolset=..." command-line
        # option form.
        if not t in sys.argv and not t in feature_toolsets:

            if debug_config:
                print "notice: [cmdline-cfg] adding toolset=%s) to the build request." % t ;
            extra_properties += "toolset=%s" % t

    return extra_properties



# Returns 'true' if the given 'project' is equal to or is a (possibly indirect)
# child to any of the projects requested to be cleaned in this build system run.
# Returns 'false' otherwise. Expects the .project-targets list to have already
# been constructed.
#
@cached
def should_clean_project(project):

    if project in project_targets:
        return True
    else:

        parent = get_manager().projects().attribute(project, "parent-module")
        if parent and parent != "user-config":
            return should_clean_project(parent)
        else:
            return False

################################################################################
#
# main()
# ------
#
################################################################################

def main():

    # FIXME: document this option.
    if "--profiling" in sys.argv:
        import cProfile
        r = cProfile.runctx('main_real()', globals(), locals(), "stones.prof")

        import pstats
        stats = pstats.Stats("stones.prof")
        stats.strip_dirs()
        stats.sort_stats('time', 'calls')
        stats.print_callers(20)
        return r
    else:
        try:
            return main_real()
        except ExceptionWithUserContext, e:
            e.report()

def main_real():

    global debug_config, out_xml

    debug_config = "--debug-configuration" in sys.argv
    out_xml = any(re.match("^--out-xml=(.*)$", a) for a in sys.argv)

    engine = Engine()

    global_build_dir = option.get("build-dir")
    manager = Manager(engine, global_build_dir)

    import b2.build.configure as configure

    if "--version" in sys.argv:
        from b2.build import version
        version.report()
        return

    # This module defines types and generator and what not,
    # and depends on manager's existence
    import b2.tools.builtin

    b2.tools.common.init(manager)

    load_configuration_files()

    # Load explicitly specified toolset modules.
    extra_properties = process_explicit_toolset_requests()

    # Load the actual project build script modules. We always load the project
    # in the current folder so 'use-project' directives have any chance of
    # being seen. Otherwise, we would not be able to refer to subprojects using
    # target ids.
    current_project = None
    projects = get_manager().projects()
    if projects.find(".", "."):
        current_project = projects.target(projects.load("."))

    # Load the default toolset module if no other has already been specified.
    if not feature.values("toolset"):

        dt = default_toolset
        dtv = None
        if default_toolset:
            dtv = default_toolset_version
        else:
            dt = "gcc"
            if os.name == 'nt':
                dt = "msvc"
            # FIXME:
            #else if [ os.name ] = MACOSX
            #{
            #    default-toolset = darwin ;
            #}

        print "warning: No toolsets are configured."
        print "warning: Configuring default toolset '%s'." % dt
        print "warning: If the default is wrong, your build may not work correctly."
        print "warning: Use the \"toolset=xxxxx\" option to override our guess."
        print "warning: For more configuration options, please consult"
        print "warning: http://boost.org/boost-build2/doc/html/bbv2/advanced/configuration.html"

        using(dt, dtv)

    # Parse command line for targets and properties. Note that this requires
    # that all project files already be loaded.
    (target_ids, properties) = build_request.from_command_line(sys.argv[1:] + extra_properties)

    # Check that we actually found something to build.
    if not current_project and not target_ids:
        get_manager().errors()("no Jamfile in current directory found, and no target references specified.")
        # FIXME:
        # EXIT

    # Flags indicating that this build system run has been started in order to
    # clean existing instead of create new targets. Note that these are not the
    # final flag values as they may get changed later on due to some special
    # targets being specified on the command line.
    clean = "--clean" in sys.argv
    cleanall = "--clean-all" in sys.argv

    # List of explicitly requested files to build. Any target references read
    # from the command line parameter not recognized as one of the targets
    # defined in the loaded Jamfiles will be interpreted as an explicitly
    # requested file to build. If any such files are explicitly requested then
    # only those files and the targets they depend on will be built and they
    # will be searched for among targets that would have been built had there
    # been no explicitly requested files.
    explicitly_requested_files = []

    # List of Boost Build meta-targets, virtual-targets and actual Jam targets
    # constructed in this build system run.
    targets = []
    virtual_targets = []
    actual_targets = []

    explicitly_requested_files = []

    # Process each target specified on the command-line and convert it into
    # internal Boost Build target objects. Detect special clean target. If no
    # main Boost Build targets were explictly requested use the current project
    # as the target.
    for id in target_ids:
        if id == "clean":
            clean = 1
        else:
            t = None
            if current_project:
                t = current_project.find(id, no_error=1)
            else:
                t = find_target(id)

            if not t:
                print "notice: could not find main target '%s'" % id
                print "notice: assuming it's a name of file to create " ;
                explicitly_requested_files.append(id)
            else:
                targets.append(t)

    if not targets:
        targets = [projects.target(projects.module_name("."))]

    # FIXME: put this BACK.

    ## if [ option.get dump-generators : : true ]
    ## {
    ##     generators.dump ;
    ## }


    # We wish to put config.log in the build directory corresponding
    # to Jamroot, so that the location does not differ depending on
    # directory where we do build.  The amount of indirection necessary
    # here is scary.
    first_project = targets[0].project()
    first_project_root_location = first_project.get('project-root')
    first_project_root_module = manager.projects().load(first_project_root_location)
    first_project_root = manager.projects().target(first_project_root_module)
    first_build_build_dir = first_project_root.build_dir()
    configure.set_log_file(os.path.join(first_build_build_dir, "config.log"))

    virtual_targets = []

    global results_of_main_targets

    # Expand properties specified on the command line into multiple property
    # sets consisting of all legal property combinations. Each expanded property
    # set will be used for a single build run. E.g. if multiple toolsets are
    # specified then requested targets will be built with each of them.
    # The expansion is being performed as late as possible so that the feature
    # validation is performed after all necessary modules (including project targets
    # on the command line) have been loaded.
    if properties:
        expanded = []
        for p in properties:
            expanded.extend(build_request.convert_command_line_element(p))

        expanded = build_request.expand_no_defaults(expanded)
    else:
        expanded = [property_set.empty()]

    # Now that we have a set of targets to build and a set of property sets to
    # build the targets with, we can start the main build process by using each
    # property set to generate virtual targets from all of our listed targets
    # and any of their dependants.
    for p in expanded:
        manager.set_command_line_free_features(property_set.create(p.free()))

        for t in targets:
            try:
                g = t.generate(p)
                if not isinstance(t, ProjectTarget):
                    results_of_main_targets.extend(g.targets())
                virtual_targets.extend(g.targets())
            except ExceptionWithUserContext, e:
                e.report()
            except Exception:
                raise

    # Convert collected virtual targets into actual raw Jam targets.
    for t in virtual_targets:
        actual_targets.append(t.actualize())

    j = option.get("jobs")
    if j:
        bjam.call("set-variable", 'PARALLELISM', j)

    k = option.get("keep-going", "true", "true")
    if k in ["on", "yes", "true"]:
        bjam.call("set-variable", "KEEP_GOING", "1")
    elif k in ["off", "no", "false"]:
        bjam.call("set-variable", "KEEP_GOING", "0")
    else:
        print "error: Invalid value for the --keep-going option"
        sys.exit()

    # The 'all' pseudo target is not strictly needed expect in the case when we
    # use it below but people often assume they always have this target
    # available and do not declare it themselves before use which may cause
    # build failures with an error message about not being able to build the
    # 'all' target.
    bjam.call("NOTFILE", "all")

    # And now that all the actual raw Jam targets and all the dependencies
    # between them have been prepared all that is left is to tell Jam to update
    # those targets.
    if explicitly_requested_files:
        # Note that this case can not be joined with the regular one when only
        # exact Boost Build targets are requested as here we do not build those
        # requested targets but only use them to construct the dependency tree
        # needed to build the explicitly requested files.
        # FIXME: add $(.out-xml)
        bjam.call("UPDATE", ["<e>%s" % x for x in explicitly_requested_files])
    elif cleanall:
        bjam.call("UPDATE", "clean-all")
    elif clean:
        manager.engine().set_update_action("common.Clean", "clean",
                                           actual_clean_targets(targets))
        bjam.call("UPDATE", "clean")
    else:
        # FIXME:
        #configure.print-configure-checks-summary ;

        if pre_build_hook:
            for h in pre_build_hook:
                h()

        bjam.call("DEPENDS", "all", actual_targets)
        ok = bjam.call("UPDATE_NOW", "all") # FIXME: add out-xml
        if post_build_hook:
            post_build_hook(ok)
        # Prevent automatic update of the 'all' target, now that
        # we have explicitly updated what we wanted.
        bjam.call("UPDATE")

    if manager.errors().count() == 0:
        return ["ok"]
    else:
        return []



#!/usr/bin/python

# Copyright 2012 Jurko Gospodnetic
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

# Tests rc toolset behaviour.

import BoostBuild


def included_resource_newer_than_rc_script():
    """
      When a .rc script file includes another resource file - the resource file
    being newer than the .rc script file should not cause the .rc script file
    to be considered old and force all of its dependents to rebuild.

    """
    toolsetName = "__myDummyResourceCompilerToolset__"

    # Used options rationale:
    #
    # -d4 & --debug-configuration
    #     Display additional information in case of test failure. In the past
    #   we have had testing system issues  causing this test to fail
    #   sporadically for which -d+3 output had been instrumental in getting to
    #   the root cause (a touched file's timestamp was not as new as it should
    #   have been).
    #
    # --ignore-site-config --user-config=
    #     Disable reading any external Boost Build configuration. This test is
    #   self sufficient so these options protect it from being adversly
    #   affected by any local (mis)configuration..
    t = BoostBuild.Tester(["-d4", "--debug-configuration",
        "--ignore-site-config", "--user-config=", "toolset=%s" % toolsetName],
        pass_d0=False, pass_toolset=False, use_test_config=False,
        translate_suffixes=False)

    # Prepare a dummy toolset so we do not get errors in case the default one
    # is not found and that we can test rc.jam functionality without having to
    # depend on the externally specified toolset actually supporting it exactly
    # the way it is required for this test, e.g. gcc toolset, under some
    # circumstances, uses a quiet action for generating its null RC targets.
    t.write(toolsetName + ".jam", """\
import feature ;
import rc ;
import type ;
local toolset-name = "%s" ;
feature.extend toolset : $(toolset-name) ;
rule init ( ) { }
rc.configure dummy-rc-command : <toolset>$(toolset-name) : <rc-type>dummy ;
module rc
{
    rule compile.resource.dummy ( targets * : sources * : properties * )
    {
        import common ;
        .TOUCH on $(targets) = [ common.file-touch-command ] ;
    }
    actions compile.resource.dummy { $(.TOUCH) "$(<)" }
}
# Make OBJ files generated by our toolset use the "obj" suffix on all
# platforms. We need to do this explicitly for <target-os> windows & cygwin to
# override the default OBJ type configuration (otherwise we would get
# 'ambiguous key' errors on those platforms).
local rule set-generated-obj-suffix ( target-os ? )
{
    type.set-generated-target-suffix OBJ : <toolset>$(toolset-name)
        <target-os>$(target-os) : obj ;
}
set-generated-obj-suffix ;
set-generated-obj-suffix windows ;
set-generated-obj-suffix cygwin ;
""" % toolsetName)

    t.write(
        toolsetName + '.py',
"""
from b2.build import feature, type as type_
from b2.manager import get_manager
from b2.tools import rc, common

MANAGER = get_manager()
ENGINE = MANAGER.engine()

toolset_name = "{0}"

feature.extend('toolset', [toolset_name])

def init(*args):
    pass

rc.configure(['dummy-rc-command'], ['<toolset>' + toolset_name], ['<rc-type>dummy'])

ENGINE.register_action(
    'rc.compile.resource.dummy',
    '''
    %s "$(<)"
    ''' % common.file_creation_command()
)

def set_generated_obj_suffix(target_os=''):
    requirements = ['<toolset>' + toolset_name]
    if target_os:
        requirements.append('<target-os>' + target_os)
    type_.set_generated_target_suffix('OBJ', requirements, 'obj')

set_generated_obj_suffix()
set_generated_obj_suffix('windows')
set_generated_obj_suffix('cygwin')
""".format(toolsetName)
    )

    # Prepare project source files.
    t.write("jamroot.jam", """\
ECHO {{{ [ modules.peek : XXX ] [ modules.peek : NOEXEC ] }}} ;
obj xxx : xxx.rc ;
""")
    t.write("xxx.rc", '1 MESSAGETABLE "xxx.bin"\n')
    t.write("xxx.bin", "foo")

    def test1(n, expect, noexec=False):
        params = ["-sXXX=%d" % n]
        if noexec:
            params.append("-n")
            params.append("-sNOEXEC=NOEXEC")
        t.run_build_system(params)
        t.expect_output_lines("*NOEXEC*", noexec)
        obj_file = "xxx_res.obj"
        t.expect_output_lines("compile.resource.dummy *%s" % obj_file, expect)
        if expect and not noexec:
            expect("bin/%s/debug/%s" % (toolsetName, obj_file))
        t.expect_nothing_more()

    def test(n, expect):
        test1(n, expect, noexec=True)
        test1(n, expect)

    test(1, t.expect_addition)
    test(2, None)
    t.touch("xxx.bin")
    test(3, t.expect_touch)
    test(4, None)

    t.cleanup()


included_resource_newer_than_rc_script()



#!/usr/bin/python

# Copyright 2013 Steven Watanabe
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

#   Tests that actions that produce multiple targets are handled
# correctly.  The rules are as follows:
#
# - If any action that updates a target is run, then the target
#   is considered to be out-of-date and all of its updating actions
#   are run in order.
# - A target is considered updated when all of its updating actions
#   have completed successfully.
# - If any updating action for a target fails, then the remaining
#   actions are skipped and the target is marked as failed.
#
# Note that this is a more thorough test case for the same
# problem that core_parallel_multifile_actions_N.py checks for.

import BoostBuild

t = BoostBuild.Tester(pass_toolset=0, pass_d0=False)

t.write("file.jam", """
actions update
{
    echo updating $(<)
}

update x1 x2 ;
update x2 x3 ;
""")

# Updating x1 should force x2 to update as well.
t.run_build_system(["-ffile.jam", "x1"], stdout="""\
...found 3 targets...
...updating 3 targets...
update x1
updating x1 x2
update x2
updating x2 x3
...updated 3 targets...
""")

# If x1 is up-to-date, we don't need to update x2,
# even though x2 is missing.
t.write("x1", "")
t.run_build_system(["-ffile.jam", "x1"], stdout="""\
...found 1 target...
""")

# Building x3 should update x1 and x2, even though
# x1 would be considered up-to-date, taken alone.
t.run_build_system(["-ffile.jam", "x3"], stdout="""\
...found 3 targets...
...updating 2 targets...
update x1
updating x1 x2
update x2
updating x2 x3
...updated 3 targets...
""")

# Updating x2 should succeed, but x3 should be skipped
t.rm("x1")
t.write("file.jam", """\
actions update
{
    echo updating $(<)
}
actions fail
{
    echo failed $(<)
    exit 1
}

update x1 x2 ;
fail x1 ;
update x1 x3 ;
update x2 ;
update x3 ;
""")

t.run_build_system(["-ffile.jam", "x3"], status=1, stdout="""\
...found 3 targets...
...updating 3 targets...
update x1
updating x1 x2
fail x1
failed x1

    echo failed x1
    exit 1

...failed fail x1...
update x2
updating x2
...failed updating 2 targets...
...updated 1 target...
""")

# Make sure that dependencies of targets that are
# updated as a result of a multifile action are
# processed correctly.
t.rm("x1")
t.write("file.jam", """\
actions update
{
    echo updating $(<)
}

update x1 ;
update x2 ;
DEPENDS x2 : x1 ;
update x2 x3 ;
""")
t.run_build_system(["-ffile.jam", "x3"], stdout="""\
...found 3 targets...
...updating 3 targets...
update x1
updating x1
update x2
updating x2
update x2
updating x2 x3
...updated 3 targets...
""")

# JAM_SEMAPHORE rules:
#
# - if two updating actions have targets that share a semaphore,
#   these actions cannot be run in parallel.
#
t.write("file.jam", """\
actions update
{
    echo updating $(<)
}

targets = x1 x2 ;
JAM_SEMAPHORE on $(targets) = <s>update_sem ;
update x1 x2 ;
""")
t.run_build_system(["-ffile.jam", "x1"], stdout="""\
...found 2 targets...
...updating 2 targets...
update x1
updating x1 x2
...updated 2 targets...
""")

# A target can appear multiple times in an action
t.write("file.jam", """\
actions update
{
    echo updating $(<)
}

update x1 x1 ;
""")
t.run_build_system(["-ffile.jam", "x1"], stdout="""\
...found 1 target...
...updating 1 target...
update x1
updating x1 x1
...updated 1 target...
""")

# Together actions should check that all the targets are the same
# before combining.
t.write("file.jam", """\
actions together update
{
    echo updating $(<) : $(>)
}

update x1 x2 : s1 ;
update x1 x2 : s2 ;

update x3 : s3 ;
update x3 x4 : s4 ;
update x4 x3 : s5 ;
DEPENDS all : x1 x2 x3 x4 ;
""")
t.run_build_system(["-ffile.jam"], stdout="""\
...found 5 targets...
...updating 4 targets...
update x1
updating x1 x2 : s1 s2
update x3
updating x3 : s3
update x3
updating x3 x4 : s4
update x4
updating x4 x3 : s5
...updated 4 targets...
""")



t.cleanup()



#!/usr/bin/python

# Copyright (C) 2013 Steven Watanabe
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

import BoostBuild
import MockToolset

t = BoostBuild.Tester(arguments=['toolset=mock', '--ignore-site-config', '--user-config='], pass_toolset=0)

MockToolset.create(t)

# Build from source
t.write("bzip2/bzlib.h", 'bzip2')
t.write("bzip2/blocksort.c", 'blocksort')

t.write("Jamroot.jam", """
path-constant here : . ;
using bzip2 : : <source>$(here)/bzip2 ;
alias bzip2 : /bzip2//bzip2 : : <link>static <link>shared ;
""")

MockToolset.set_expected(t, '''
source_file('blocksort.c', 'blocksort')
action('-c -x c -I./bzip2 -o $blocksort.o $blocksort.c')
action('--dll $blocksort.o -o $bz2.so')
action('--archive $blocksort.o -o $bz2.a')
''')

t.run_build_system()
t.expect_addition('bin/standalone/bzip2/mock/debug/bz2.dll')
t.expect_addition('bin/standalone/bzip2/mock/debug/link-static/bz2.lib')

t.rm('bzip2')

# Generic definitions that aren't configuration specific
common_stuff = '''
source_file('test.cpp', 'test.cpp')
source_file('main.cpp', 'int main() {}')
source_file('bzlib.h.cpp', '#include <bzlib.h>\\n')
action('-c -x c++ $main.cpp -o $main.o')
'''
t.write('test.cpp', 'test.cpp')

# Default initialization - static library
t.rm('bin')
t.write("Jamroot.jam", """
path-constant here : . ;
using bzip2 ;
exe test : test.cpp /bzip2//bzip2 : : <link>static <link>shared ;
""")

MockToolset.set_expected(t, common_stuff + '''
action('$main.o --static-lib=bz2 -o $config.exe')
action('-c -x c++ $bzlib.h.cpp -o $bzlib.h.o')
action('-c -x c++ $test.cpp -o $test.o')
action('$test.o --static-lib=bz2 -o $test')
''')
t.run_build_system()
t.expect_addition('bin/mock/debug/test.exe')
t.expect_addition('bin/mock/debug/link-static/test.exe')

# Default initialization - shared library
t.rm('bin')
t.write("Jamroot.jam", """
path-constant here : . ;
using bzip2 ;
exe test : test.cpp /bzip2//bzip2 : : <link>static <link>shared ;
""")

MockToolset.set_expected(t, common_stuff + '''
action('$main.o --shared-lib=bz2 -o $config.exe')
action('-c -x c++ $bzlib.h.cpp -o $bzlib.h.o')
action('-c -x c++ $test.cpp -o $test.o')
action('$test.o --shared-lib=bz2 -o $test')
''')
t.run_build_system()
t.expect_addition('bin/mock/debug/test.exe')
t.expect_addition('bin/mock/debug/link-static/test.exe')

# Initialization in explicit location - static library
t.rm('bin')
t.write("Jamroot.jam", """
path-constant here : . ;
using bzip2 : : <name>mybzlib <include>$(here)/bzip2 <search>$(here)/bzip2 ;
exe test : test.cpp /bzip2//bzip2 : : <link>static <link>shared ;
""")

t.write('bzip2/bzlib.h', 'bzip2')

MockToolset.set_expected(t, common_stuff + '''
action('$main.o -L./bzip2 --static-lib=mybzlib -o $config.exe')
action('-c -x c++ $test.cpp -I./bzip2 -o $test.o')
action('$test.o -L./bzip2 --static-lib=mybzlib -o $test')
''')
t.run_build_system()
t.expect_addition('bin/mock/debug/test.exe')
t.expect_addition('bin/mock/debug/link-static/test.exe')

# Initialization in explicit location - shared library
t.rm('bin')
t.write("Jamroot.jam", """
path-constant here : . ;
using bzip2 : : <name>mybzlib <include>$(here)/bzip2 <search>$(here)/bzip2 ;
exe test : test.cpp /bzip2//bzip2 : : <link>static <link>shared ;
""")

MockToolset.set_expected(t, common_stuff + '''
action('$main.o -L./bzip2 --shared-lib=mybzlib -o $config.exe')
action('-c -x c++ $test.cpp -I./bzip2 -o $test.o')
action('$test.o -L./bzip2 --shared-lib=mybzlib -o $test')
''')
t.run_build_system()
t.expect_addition('bin/mock/debug/test.exe')
t.expect_addition('bin/mock/debug/link-static/test.exe')

t.cleanup()



#!/usr/bin/python
#
# Copyright (c) 2008 Steven Watanabe
#
# Distributed under the Boost Software License, Version 1.0. (See
# accompanying file LICENSE_1_0.txt) or copy at
# http://www.boost.org/LICENSE_1_0.txt)

import BoostBuild

def test_default_order():
    tester = BoostBuild.Tester(use_test_config=False)
    tester.write("jamroot.jam", """

    import order ;
    import "class" : new ;

    obj test : test.cpp : <include>b <include>a ;
    """)

    tester.write("test.cpp", """
    #include <test.hpp>
    int main() { f(); }
    """)
    
    tester.write("a/test.hpp", """
    void f();
    """)
    
    tester.write("b/test.hpp", """
    """)
    
    tester.run_build_system()
    
    tester.expect_addition("bin/$toolset/debug*/test.obj")
    
    # Check that the dependencies are correct
    tester.touch("a/test.hpp")
    tester.run_build_system()
    tester.expect_touch("bin/$toolset/debug*/test.obj")
    tester.expect_nothing_more()
    
    tester.touch("b/test.hpp")
    tester.run_build_system()
    tester.expect_nothing_more()

    tester.cleanup()

def test_default_order_mixed():
    tester = BoostBuild.Tester(use_test_config=False)
    tester.write("jamroot.jam", """

    import order ;
    import "class" : new ;

    obj test : test.cpp : <include>b <include>a <include>c&&d ;
    """)

    tester.write("test.cpp", """
    #include <test.hpp>
    int main() { f(); }
    """)
    
    tester.write("a/test.hpp", """
    void f();
    """)
    
    tester.write("b/test.hpp", """
    """)
    
    tester.run_build_system()
    
    tester.expect_addition("bin/$toolset/debug*/test.obj")
    
    # Check that the dependencies are correct
    tester.touch("a/test.hpp")
    tester.run_build_system()
    tester.expect_touch("bin/$toolset/debug*/test.obj")
    tester.expect_nothing_more()
    
    tester.touch("b/test.hpp")
    tester.run_build_system()
    tester.expect_nothing_more()

    tester.cleanup()

def test_basic():
    tester = BoostBuild.Tester(use_test_config=False)
    tester.write("jamroot.jam", """
    obj test : test.cpp : <include>a&&b ;
    """)

    tester.write("test.cpp", """
    #include <test1.hpp>
    #include <test2.hpp>
    int main() {}
    """)

    tester.write("a/test1.hpp", """
    """)

    tester.write("b/test2.hpp", """
    """)

    tester.run_build_system()

    tester.expect_addition("bin/$toolset/debug*/test.obj")

    # Check that the dependencies are correct
    tester.touch("a/test1.hpp")
    tester.run_build_system()
    tester.expect_touch("bin/$toolset/debug*/test.obj")

    tester.touch("b/test2.hpp")
    tester.run_build_system()
    tester.expect_touch("bin/$toolset/debug*/test.obj")

    tester.cleanup()

def test_order1():
    t = BoostBuild.Tester(use_test_config=False)
    t.write("jamroot.jam", """
    obj test : test.cpp : <include>a&&b ;
    """)
    t.write("test.cpp", """
    #include <test.h>
    int main() {}
    """)
    t.write("a/test.h", """
    """)
    t.write("b/test.h", """
    #error should find a/test.h
    """)
    t.run_build_system()

    t.touch("a/test.h")
    t.run_build_system()
    t.expect_touch("bin/$toolset/debug*/test.obj")
    t.expect_nothing_more()

    t.touch("b/test.h")
    t.run_build_system()
    t.expect_nothing_more()

    t.cleanup()

def test_order2():
    t = BoostBuild.Tester(use_test_config=False)
    t.write("jamroot.jam", """
    obj test : test.cpp : <include>b&&a ;
    """)
    t.write("test.cpp", """
    #include <test.h>
    int main() {}
    """)
    t.write("a/test.h", """
    #error should find b/test.h
    """)
    t.write("b/test.h", """
    """)
    t.run_build_system()

    t.touch("a/test.h")
    t.run_build_system()
    t.expect_nothing_more()

    t.touch("b/test.h")
    t.run_build_system()
    t.expect_touch("bin/$toolset/debug*/test.obj")
    t.expect_nothing_more()

    t.cleanup()

def test_order_graph():
    t = BoostBuild.Tester(use_test_config=False)
    t.write("jamroot.jam", """
    obj test : test.cpp :
        <include>b&&a
        <include>c&&b
        <include>a
        <include>c
        <include>b
        <include>e&&b&&d
      ;
    """)
    t.write("test.cpp", """
    #include <test1.h>
    #include <test2.h>
    #include <test3.h>
    #include <test4.h>
    int main() {}
    """)
    t.write("b/test1.h", "")
    t.write("a/test1.h", "#error should find b/test1.h\n")

    t.write("c/test2.h", "")
    t.write("b/test2.h", "#error should find c/test2.h\n")

    t.write("e/test3.h", "")
    t.write("b/test3.h", "#error should find e/test3.h\n")

    t.write("b/test4.h", "")
    t.write("d/test4.h", "#error should find b/test4.h\n")

    t.run_build_system()
    t.expect_addition("bin/$toolset/debug*/test.obj")

    t.touch("b/test1.h")
    t.run_build_system()
    t.expect_touch("bin/$toolset/debug*/test.obj")
    t.expect_nothing_more()

    t.touch("a/test1.h")
    t.run_build_system()
    t.expect_nothing_more()

    t.touch("c/test2.h")
    t.run_build_system()
    t.expect_touch("bin/$toolset/debug*/test.obj")
    t.expect_nothing_more()

    t.touch("b/test2.h")
    t.run_build_system()
    t.expect_nothing_more()

    t.touch("e/test3.h")
    t.run_build_system()
    t.expect_touch("bin/$toolset/debug*/test.obj")
    t.expect_nothing_more()

    t.touch("b/test3.h")
    t.run_build_system()
    t.expect_nothing_more()

    t.touch("b/test4.h")
    t.run_build_system()
    t.expect_touch("bin/$toolset/debug*/test.obj")
    t.expect_nothing_more()

    t.touch("d/test4.h")
    t.run_build_system()
    t.expect_nothing_more()

    t.cleanup()

test_default_order()
test_default_order_mixed()
test_basic()
test_order1()
test_order2()
test_order_graph()



#!/usr/bin/python

# Copyright (C) 2008. Jurko Gospodnetic
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

# Tests for the Boost Jam builtin SORT rule.

import BoostBuild


###############################################################################
#
# testSORTCorrectness()
# ---------------------
#
###############################################################################

def testSORTCorrectness():
    """Testing that Boost Jam's SORT builtin rule actually sorts correctly."""
    t = BoostBuild.Tester(["-ftest.jam", "-d1"], pass_toolset=False,
        use_test_config=False)

    t.write("test.jam", """\
NOCARE all ;
source-data = 1 8 9 2 7 3 4 7 1 27 27 9 98 98 1 1 4 5 6 2 3 4 8 1 -2 -2 0 0 0 ;
target-data = -2 -2 0 0 0 1 1 1 1 1 2 2 27 27 3 3 4 4 4 5 6 7 7 8 8 9 9 98 98 ;
ECHO "starting up" ;
sorted-data = [ SORT $(source-data) ] ;
ECHO "done" ;
if $(sorted-data) != $(target-data)
{
    ECHO "Source       :" $(source-data) ;
    ECHO "Expected     :" $(target-data) ;
    ECHO "SORT returned:" $(sorted-data) ;
    EXIT "SORT error" : -2 ;
}
""")

    t.run_build_system()
    t.expect_output_lines("starting up")
    t.expect_output_lines("done")
    t.expect_output_lines("SORT error", False)

    t.cleanup()


###############################################################################
#
# testSORTDuration()
# ------------------
#
###############################################################################

def testSORTDuration():
    """
      Regression test making sure Boost Jam's SORT builtin rule does not get
    quadratic behaviour again in this use case.

    """
    t = BoostBuild.Tester(["-ftest.jam", "-d1"], pass_toolset=False,
        use_test_config=False)

    f = open(t.workpath("test.jam"), "w")
    print >> f, "data = "
    for i in range(0, 20000):
        if i % 2:
            print >> f, '"aaa"'
        else:
            print >> f, '"bbb"'
    print >> f, """;

ECHO "starting up" ;
sorted = [ SORT $(data) ] ;
ECHO "done" ;
NOCARE all ;
"""
    f.close()

    t.run_build_system(expected_duration=1)
    t.expect_output_lines("starting up")
    t.expect_output_lines("done")

    t.cleanup()


###############################################################################
#
# main()
# ------
#
###############################################################################

testSORTCorrectness()
testSORTDuration()



#!/usr/bin/python

# Copyright 2004 Vladimir Prus
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

# This checks that Boost.Build does not reorder <include> properties
# lexicographically.

import BoostBuild

t = BoostBuild.Tester()

t.write("a.cpp", """
#include <a.h>
int main() { foo(); }
""")

t.write("jamroot.jam", """
exe a : a.cpp : <include>d2 <include>d1 ;
""")

t.write("d1/a.h", """
""")

t.write("d2/a.h", """
inline void foo() {}
""")

t.run_build_system()
t.expect_addition("bin/$toolset/debug*/a.exe")

t.cleanup()



#!/usr/bin/python

# Copyright 2008, 2012 Jurko Gospodnetic
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

# Tests that generators get selected correctly.
#
# We do not use the internal C++-compiler CPP --> OBJ generator to avoid
# problems with specific compilers or their configurations, e.g. IBM's AIX test
# runner 'AIX Version 5.3 TL7 SP5 (5300-07-05-0831)' using the 'IBM XL C/C++
# for AIX, V12.1 (Version: 12.01.0000.0000)' reporting errors when run with a
# source file whose suffix is not '.cpp'.

import BoostBuild


###############################################################################
#
# test_generator_added_after_already_building_a_target_of_its_target_type()
# -------------------------------------------------------------------------
#
###############################################################################

def test_generator_added_after_already_building_a_target_of_its_target_type():
    """
      Regression test for a Boost Build bug causing it to not use a generator
    if it got added after already building a target of its target type.

    """
    t = BoostBuild.Tester()

    t.write("dummy.cpp", "void f() {}\n")

    t.write("jamroot.jam", """\
import common ;
import generators ;
import type ;
type.register MY_OBJ : my_obj ;
generators.register-standard common.copy : CPP : MY_OBJ ;

# Building this dummy target must not cause a later defined CPP target type
# generator not to be recognized as viable.
my-obj dummy : dummy.cpp ;
alias the-other-obj : Other//other-obj ;
""")

    t.write("Other/source.extension", "A dummy source file.")

    t.write("Other/mygen.jam", """\
import common ;
import generators ;
import type ;
type.register MY_TYPE : extension ;
generators.register-standard $(__name__).generate-a-cpp-file : MY_TYPE : CPP ;
rule generate-a-cpp-file { ECHO Generating a CPP file... ; }
CREATE-FILE = [ common.file-creation-command ] ;
actions generate-a-cpp-file { $(CREATE-FILE) "$(<)" }
""")

    t.write("Other/mygen.py", """\
import b2.build.generators as generators
import b2.build.type as type

from b2.manager import get_manager

import os

type.register('MY_TYPE', ['extension'])
generators.register_standard('mygen.generate-a-cpp-file', ['MY_TYPE'], ['CPP'])
if os.name == 'nt':
    action = 'echo void g() {} > "$(<)"'
else:
    action = 'echo "void g() {}" > "$(<)"'
def f(*args):
    print "Generating a CPP file..."

get_manager().engine().register_action("mygen.generate-a-cpp-file", action,
    function=f)
""")

    t.write("Other/jamfile.jam", """\
import mygen ;
my-obj other-obj : source.extension ;
""")

    t.run_build_system()
    t.expect_output_lines("Generating a CPP file...")
    t.expect_addition("bin/$toolset/debug*/dummy.my_obj")
    t.expect_addition("Other/bin/$toolset/debug*/other-obj.cpp")
    t.expect_addition("Other/bin/$toolset/debug*/other-obj.my_obj")
    t.expect_nothing_more()

    t.cleanup()


###############################################################################
#
# test_using_a_derived_source_type_created_after_generator_already_used()
# -----------------------------------------------------------------------
#
###############################################################################

def test_using_a_derived_source_type_created_after_generator_already_used():
    """
      Regression test for a Boost Build bug causing it to not use a generator
    with a source type derived from one of the generator's sources but created
    only after already using the generateor.

    """
    t = BoostBuild.Tester()

    t.write("dummy.xxx", "Hello. My name is Peter Pan.\n")

    t.write("jamroot.jam", """\
import common ;
import generators ;
import type ;
type.register XXX : xxx ;
type.register YYY : yyy ;
generators.register-standard common.copy : XXX : YYY ;

# Building this dummy target must not cause a later defined XXX2 target type not
# to be recognized as a viable source type for building YYY targets.
yyy dummy : dummy.xxx ;
alias the-test-output : Other//other ;
""")

    t.write("Other/source.xxx2", "Hello. My name is Tinkerbell.\n")

    t.write("Other/jamfile.jam", """\
import type ;
type.register XXX2 : xxx2 : XXX ;
# We are careful not to do anything between defining our new XXX2 target type
# and using the XXX --> YYY generator that could potentially cover the Boost
# Build bug by clearing its internal viable source target type state.
yyy other : source.xxx2 ;
""")

    t.run_build_system()
    t.expect_addition("bin/$toolset/debug*/dummy.yyy")
    t.expect_addition("Other/bin/$toolset/debug*/other.yyy")
    t.expect_nothing_more()

    t.cleanup()


###############################################################################
#
# main()
# ------
#
###############################################################################

test_generator_added_after_already_building_a_target_of_its_target_type()
test_using_a_derived_source_type_created_after_generator_already_used()



#!/usr/bin/python

# (c) Copyright Juergen Hunold 2008
# Use, modification, and distribution are subject to the
# Boost Software License, Version 1.0. (See accompanying file
# LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

import BoostBuild
import os

# Run test in real directory in order to find Boost.Test via Boost Top-Level
# Jamroot.
qt4_dir = os.getcwd() + "/qt4"

t = BoostBuild.Tester(workdir=qt4_dir)

t.run_build_system()

t.cleanup()



#!/usr/bin/python

# Copyright 2011 Steven Watanabe
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

import BoostBuild
import os


def basic():
    t = BoostBuild.Tester(pass_toolset=0, pass_d0=False)

    t.write("file.jam", """\
actions do-print
{
    echo updating $(<)
}

NOTFILE target1 ;
ALWAYS target1 ;
do-print target1 ;

UPDATE_NOW target1 ;

DEPENDS all : target1 ;
""")

    t.run_build_system(["-ffile.jam"], stdout="""\
...found 1 target...
...updating 1 target...
do-print target1
updating target1
...updated 1 target...
...found 1 target...
""")

    t.cleanup()


def ignore_minus_n():
    t = BoostBuild.Tester(pass_toolset=0, pass_d0=False)

    t.write("file.jam", """\
actions do-print
{
    echo updating $(<)
}

NOTFILE target1 ;
ALWAYS target1 ;
do-print target1 ;

UPDATE_NOW target1 : : ignore-minus-n ;

DEPENDS all : target1 ;
""")

    t.run_build_system(["-ffile.jam", "-n"], stdout="""\
...found 1 target...
...updating 1 target...
do-print target1

    echo updating target1

updating target1
...updated 1 target...
...found 1 target...
""")

    t.cleanup()


def failed_target():
    t = BoostBuild.Tester(pass_toolset=0, pass_d0=False)

    t.write("file.jam", """\
actions fail
{
    exit 1
}

NOTFILE target1 ;
ALWAYS target1 ;
fail target1 ;

actions do-print
{
    echo updating $(<)
}

NOTFILE target2 ;
do-print target2 ;
DEPENDS target2 : target1 ;

UPDATE_NOW target1 : : ignore-minus-n ;

DEPENDS all : target1 target2 ;
""")

    t.run_build_system(["-ffile.jam", "-n"], stdout="""\
...found 1 target...
...updating 1 target...
fail target1

    exit 1

...failed fail target1...
...failed updating 1 target...
...found 2 targets...
...updating 1 target...
do-print target2

    echo updating target2

...updated 1 target...
""")

    t.cleanup()


def missing_target():
    t = BoostBuild.Tester(pass_toolset=0, pass_d0=False)

    t.write("file.jam", """\
actions do-print
{
    echo updating $(<)
}

NOTFILE target2 ;
do-print target2 ;
DEPENDS target2 : target1 ;

UPDATE_NOW target1 : : ignore-minus-n ;

DEPENDS all : target1 target2 ;
""")

    t.run_build_system(["-ffile.jam", "-n"], status=1, stdout="""\
don't know how to make target1
...found 1 target...
...can't find 1 target...
...found 2 targets...
...can't make 1 target...
""")

    t.cleanup()


def build_once():
    """
      Make sure that if we call UPDATE_NOW with ignore-minus-n, the target gets
    updated exactly once regardless of previous calls to UPDATE_NOW with -n in
    effect.

    """
    t = BoostBuild.Tester(pass_toolset=0, pass_d0=False)

    t.write("file.jam", """\
actions do-print
{
    echo updating $(<)
}

NOTFILE target1 ;
ALWAYS target1 ;
do-print target1 ;

UPDATE_NOW target1 ;
UPDATE_NOW target1 : : ignore-minus-n ;
UPDATE_NOW target1 : : ignore-minus-n ;

DEPENDS all : target1 ;
""")

    t.run_build_system(["-ffile.jam", "-n"], stdout="""\
...found 1 target...
...updating 1 target...
do-print target1

    echo updating target1

...updated 1 target...
do-print target1

    echo updating target1

updating target1
...updated 1 target...
...found 1 target...
""")

    t.cleanup()


def return_status():
    """
    Make sure that UPDATE_NOW returns a failure status if
    the target failed in a previous call to UPDATE_NOW
    """
    t = BoostBuild.Tester(pass_toolset=0, pass_d0=False)

    t.write("file.jam", """\
actions fail
{
    exit 1
}

NOTFILE target1 ;
ALWAYS target1 ;
fail target1 ;

ECHO update1: [ UPDATE_NOW target1 ] ;
ECHO update2: [ UPDATE_NOW target1 ] ;

DEPENDS all : target1 ;
""")

    t.run_build_system(["-ffile.jam"], status=1, stdout="""\
...found 1 target...
...updating 1 target...
fail target1

    exit 1

...failed fail target1...
...failed updating 1 target...
update1:
update2:
...found 1 target...
""")

    t.cleanup()


def save_restore():
    """Tests that ignore-minus-n and ignore-minus-q are
    local to the call to UPDATE_NOW"""
    t = BoostBuild.Tester(pass_toolset=0, pass_d0=False)

    t.write("actions.jam", """\
rule fail
{
    NOTFILE $(<) ;
    ALWAYS $(<) ;
}
actions fail
{
    exit 1
}

rule pass
{
    NOTFILE $(<) ;
    ALWAYS $(<) ;
}
actions pass
{
    echo updating $(<)
}
""")
    t.write("file.jam", """
include actions.jam ;
fail target1 ;
fail target2 ;
UPDATE_NOW target1 target2 : : $(IGNORE_MINUS_N) : $(IGNORE_MINUS_Q) ;
fail target3 ;
fail target4 ;
UPDATE_NOW target3 target4 ;
UPDATE ;
""")
    t.run_build_system(['-n', '-sIGNORE_MINUS_N=1', '-ffile.jam'],
                       stdout='''...found 2 targets...
...updating 2 targets...
fail target1

    exit 1

...failed fail target1...
fail target2

    exit 1

...failed fail target2...
...failed updating 2 targets...
...found 2 targets...
...updating 2 targets...
fail target3

    exit 1

fail target4

    exit 1

...updated 2 targets...
''')

    t.run_build_system(['-q', '-sIGNORE_MINUS_N=1', '-ffile.jam'],
                       status=1, stdout='''...found 2 targets...
...updating 2 targets...
fail target1

    exit 1

...failed fail target1...
...failed updating 1 target...
...found 2 targets...
...updating 2 targets...
fail target3

    exit 1

...failed fail target3...
...failed updating 1 target...
''')

    t.run_build_system(['-n', '-sIGNORE_MINUS_Q=1', '-ffile.jam'],
                       stdout='''...found 2 targets...
...updating 2 targets...
fail target1

    exit 1

fail target2

    exit 1

...updated 2 targets...
...found 2 targets...
...updating 2 targets...
fail target3

    exit 1

fail target4

    exit 1

...updated 2 targets...
''')

    t.run_build_system(['-q', '-sIGNORE_MINUS_Q=1', '-ffile.jam'],
                       status=1, stdout='''...found 2 targets...
...updating 2 targets...
fail target1

    exit 1

...failed fail target1...
fail target2

    exit 1

...failed fail target2...
...failed updating 2 targets...
...found 2 targets...
...updating 2 targets...
fail target3

    exit 1

...failed fail target3...
...failed updating 1 target...
''')

    t.cleanup()


basic()
ignore_minus_n()
failed_target()
missing_target()
build_once()
return_status()
save_restore()



#!/usr/bin/python

# Copyright 2014-2015 Steven Watanabe
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

# Tests the link-directory rule used to create the
# common boost/ directory in the new git layout.

import BoostBuild

def ignore_config(t):
    """These files are created by the configuration logic in link.jam
    They may or may not exist, depending on the system."""
    t.ignore("bin/symlink/test-hardlink")
    t.ignore("bin/test-hardlink-source")
    t.ignore("bin/test-symlink")
    t.ignore("bin/test-symlink-source")

def test_basic():
    """Test creation of a single link"""
    t = BoostBuild.Tester()
    t.write("jamroot.jam", """\
    import link ;
    link-directory dir1-link : src/dir1/include : <location>. ;
    """)

    t.write("src/dir1/include/file1.h", "file1")

    t.run_build_system()

    t.expect_addition("include/file1.h")
    t.expect_content("include/file1.h", "file1")
    ignore_config(t)
    t.expect_nothing_more()
    t.cleanup()

def test_merge_two():
    """Test merging two directories"""
    t = BoostBuild.Tester()
    t.write("jamroot.jam", """\
    import link ;
    link-directory dir1-link : src/dir1/include : <location>. ;
    link-directory dir2-link : src/dir2/include : <location>. ;
    """)

    t.write("src/dir1/include/file1.h", "file1")
    t.write("src/dir2/include/file2.h", "file2")

    t.run_build_system()

    t.expect_addition("include/file1.h")
    t.expect_content("include/file1.h", "file1")
    t.expect_addition("include/file2.h")
    t.expect_content("include/file2.h", "file2")
    ignore_config(t)
    t.expect_nothing_more()
    t.cleanup()

def test_merge_existing(group1, group2):
    """Test adding a link when a different symlink already exists"""
    t = BoostBuild.Tester()
    t.write("jamroot.jam", """\
    import link ;
    link-directory dir1-link : src/dir1/include : <location>. ;
    link-directory dir2-link : src/dir2/include : <location>. ;
    """)

    t.write("src/dir1/include/file1.h", "file1")
    t.write("src/dir2/include/file2.h", "file2")

    t.run_build_system(group1)

    if "dir1-link" in group1:
        t.expect_addition("include/file1.h")
        t.expect_content("include/file1.h", "file1")
    if "dir2-link" in group1:
        t.expect_addition("include/file2.h")
        t.expect_content("include/file2.h", "file2")
    ignore_config(t)
    t.expect_nothing_more()

    t.run_build_system(group2)

    if "dir1-link" in group2:
        if "dir1-link" not in group1:
            t.expect_addition("include/file1.h")
        else:
            # When a directory is split the link needs to be recreated.
            # On Windows, the test system checks the mod time of the
            # link rather than the link target, so this may be seen as
            # an update.
            t.ignore_touch("include/file1.h")
        t.expect_content("include/file1.h", "file1")
    else:
        t.ignore_removal("include/file1.h")
        
    if "dir2-link" in group2:
        if "dir2-link" not in group1:
            t.expect_addition("include/file2.h")
        else:
            t.ignore_touch("include/file2.h")
        t.expect_content("include/file2.h", "file2")
    else:
        t.ignore_removal("include/file2.h")
    ignore_config(t)
    t.expect_nothing_more()

    t.cleanup()

def test_merge_existing_all():
    test_merge_existing(["dir1-link"], ["dir2-link"])
    test_merge_existing(["dir2-link"], ["dir1-link"])
    test_merge_existing(["dir1-link"], ["dir1-link", "dir2-link"])
    test_merge_existing(["dir2-link"], ["dir1-link", "dir2-link"])

def test_merge_recursive():
    "Test merging several directories including common prefixes"
    t = BoostBuild.Tester()
    t.write("jamroot.jam", """\
    import link ;
    link-directory dir1-link : src/dir1/include : <location>. ;
    link-directory dir2-link : src/dir2/include : <location>. ;
    link-directory dir3-link : src/dir3/include : <location>. ;
    """)

    t.write("src/dir1/include/file1.h", "file1")
    t.write("src/dir2/include/file2.h", "file2")
    t.write("src/dir2/include/nested/file3.h", "file3")
    t.write("src/dir3/include/nested/file4.h", "file4")

    t.run_build_system()

    t.expect_addition("include/file1.h")
    t.expect_content("include/file1.h", "file1")
    t.expect_addition("include/file2.h")
    t.expect_content("include/file2.h", "file2")
    t.expect_addition("include/nested/file3.h")
    t.expect_content("include/nested/file3.h", "file3")
    t.expect_addition("include/nested/file4.h")
    t.expect_content("include/nested/file4.h", "file4")
    ignore_config(t)
    t.expect_nothing_more()

    t.cleanup()

def test_merge_recursive_existing(group1, group2):
    "Test merging several directories including common prefixes."
    t = BoostBuild.Tester()
    t.write("jamroot.jam", """\
    import link ;
    link-directory dir1-link : src/dir1/include : <location>. ;
    link-directory dir2-link : src/dir2/include : <location>. ;
    link-directory dir3-link : src/dir3/include : <location>. ;
    link-directory dir4-link : src/dir4/include : <location>. ;
    link-directory dir5-link : src/dir5/include : <location>. ;
    """)

    t.write("src/dir1/include/file1.h", "file1")
    t.write("src/dir2/include/nested/file2.h", "file2")
    t.write("src/dir3/include/nested/file3.h", "file3")
    t.write("src/dir4/include/nested/xxx/yyy/file4.h", "file4")
    t.write("src/dir5/include/nested/xxx/yyy/file5.h", "file5")

    t.run_build_system(group1)
    t.run_build_system(group2 + ["-d+12"])

    def check_file(target, file):
        if target in group2:
            if target in group1:
                t.ignore_touch(file)
            else:
                t.expect_addition(file)

    check_file("dir1-link", "include/file1.h")
    check_file("dir2-link", "include/nested/file2.h")
    check_file("dir3-link", "include/nested/file3.h")
    check_file("dir4-link", "include/nested/xxx/yyy/file4.h")
    check_file("dir5-link", "include/nested/xxx/yyy/file5.h")
    ignore_config(t)
    t.expect_nothing_more()

    t.cleanup()

def test_merge_recursive_existing_all():
    # These should create a link
    test_merge_recursive_existing(["dir2-link"], ["dir2-link", "dir1-link"])
    test_merge_recursive_existing(["dir2-link"], ["dir1-link", "dir2-link"])
    # These should create a directory
    test_merge_recursive_existing(["dir2-link"], ["dir2-link", "dir3-link"])
    test_merge_recursive_existing(["dir2-link"], ["dir3-link", "dir2-link"])
    # It should work even if we have to create many intermediate subdirectories
    test_merge_recursive_existing(["dir4-link"], ["dir4-link", "dir5-link"])
    test_merge_recursive_existing(["dir4-link"], ["dir5-link", "dir4-link"])

def test_include_scan():
    """Make sure that the #include scanner finds the headers"""
    t = BoostBuild.Tester()
    t.write("jamroot.jam", """\
    import link ;
    link-directory dir1-link : src/dir1/include : <location>. ;
    link-directory dir2-link : src/dir2/include : <location>. ;
    obj test : test.cpp :
        <include>include
        <implicit-dependency>dir1-link
        <implicit-dependency>dir2-link ;
    """)

    t.write("src/dir1/include/file1.h", "#include <file2.h>\n")
    t.write("src/dir2/include/file2.h", "int f();\n")
    t.write("test.cpp", """\
    #include <file1.h>
    int main() { f(); }
    """);

    t.run_build_system(["test"])

    t.expect_addition("bin/$toolset/debug*/test.obj")

    t.run_build_system()
    t.expect_nothing_more()

    t.cleanup()

def test_include_scan_merge_existing():
    """Make sure that files are replaced if needed when merging in
    a new directory"""
    t = BoostBuild.Tester()

    t.write("jamroot.jam", """\
    import link ;
    link-directory dir1-link : src/dir1/include : <location>. ;
    link-directory dir2-link : src/dir2/include : <location>. ;
    obj test : test.cpp :
        <include>include
        <implicit-dependency>dir1-link
        <implicit-dependency>dir2-link ;
    """)

    t.write("src/dir1/include/file1.h", "int f();")
    t.write("src/dir2/include/file2.h", "#include <file1.h>")
    t.write("test.cpp", """\
    #include <file2.h>
    int main() { f(); }
    """)

    t.run_build_system(["dir2-link"])

    t.run_build_system(["test"])
    t.expect_addition("include/file1.h")
    t.expect_addition("bin/$toolset/debug*/test.obj")
    t.ignore_touch("include/file2.h")
    t.expect_nothing_more()

    t.cleanup()

def test_update_file_link(params1, params2):
    """Tests the behavior of updates when changing the link mode.
    The link needs to be updated iff the original was a copy."""
    t = BoostBuild.Tester()

    t.write("jamroot.jam", """\
    import link ;
    import project ;
    import property-set ;
    import modules ;

    if --no-symlinks in [ modules.peek : ARGV ]
    {
        modules.poke link : .can-symlink : false ;
    }

    if --no-hardlinks in [ modules.peek : ARGV ]
    {
        modules.poke link : .can-hardlink : false ;
    }

    .project = [ project.current ] ;
    .has-files = [ glob include/file1.h ] ;
    
    rule can-link ( properties * ) {
        if ( ! [ link.can-symlink $(.project) : [ property-set.empty ] ] ) &&
           ( ! [ link.can-hardlink $(.project) : [ property-set.empty ] ] )
        {
            ECHO links unsupported ;
        }
    }

    # Use two directories so that we link to individual files.
    link-directory dir1-link : src/dir1/include : <location>. ;
    link-directory dir2-link : src/dir2/include : <location>. ;
    alias check-linking : : <conditional>@can-link ;
    """)
    t.write("src/dir1/include/file1.h", "file1")
    t.write("src/dir2/include/file2.h", "file2")

    t.run_build_system(params1)
    ignore_config(t)
    t.expect_addition("include/file1.h")
    t.expect_addition("include/file2.h")
    t.expect_nothing_more()

    using_links = "links unsupported" not in t.stdout()

    t.touch("src/dir1/include/file1.h")

    t.run_build_system(params2)
    if not using_links: t.expect_touch("include/file1.h")
    ignore_config(t)
    t.expect_nothing_more()

    t.cleanup()

def test_update_file_link_all():
    """Test all nine possible combinations of two runs."""
    possible_args = [[], ["--no-symlinks"], ["--no-symlinks", "--no-hardlinks"]]
    for arg1 in possible_args:
        for arg2 in possible_args:
            test_update_file_link(arg1, arg2)

def test_error_duplicate():
    """Test that linking a single file from
    multiple sources causes a hard error."""
    t = BoostBuild.Tester()

    t.write("jamroot.jam", """\
    import link ;
    link-directory dir1-link : src/dir1/include : <location>. ;
    link-directory dir2-link : src/dir2/include : <location>. ;
    """)

    t.write("src/dir1/include/file1.h", "file1")
    t.write("src/dir2/include/file1.h", "file2")

    t.run_build_system(status=1)
    t.expect_output_lines(
        ["error: Cannot create link include/file1.h to src/dir2/include/file1.h.",
         "error: Link previously defined to another file, src/dir1/include/file1.h."])

    t.cleanup()

test_basic()
test_merge_two()
test_merge_existing_all()
test_merge_recursive()
test_merge_recursive_existing_all()
test_include_scan()
test_include_scan_merge_existing()
test_update_file_link_all()
test_error_duplicate()



#!/usr/bin/python

# Copyright 2003 Vladimir Prus
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

import BoostBuild

t = BoostBuild.Tester(use_test_config=False)

t.write("jamroot.jam", """\
exe hello : hello.cpp ;
exe hello2 : hello.cpp ;
explicit hello2 ;
""")

t.write("hello.cpp", "int main() {}\n")

t.run_build_system()
t.ignore("*.tds")
t.expect_addition(BoostBuild.List("bin/$toolset/debug*/hello") * \
    [".exe", ".obj"])
t.expect_nothing_more()

t.run_build_system(["hello2"])
t.expect_addition("bin/$toolset/debug*/hello2.exe")

t.rm(".")


# Test that 'explicit' used in a helper rule applies to the current project, and
# not to the Jamfile where the helper rule is defined.
t.write("jamroot.jam", """\
rule myinstall ( name : target )
{
    install $(name)-bin : $(target) ;
    explicit $(name)-bin ;
    alias $(name) : $(name)-bin ;
}
""")

t.write("sub/a.cpp", "\n")
t.write("sub/jamfile.jam", "myinstall dist : a.cpp ;")

t.run_build_system(subdir="sub")
t.expect_addition("sub/dist-bin/a.cpp")

t.rm("sub/dist-bin")

t.write("sub/jamfile.jam", """\
myinstall dist : a.cpp ;
explicit dist ;
""")

t.run_build_system(subdir="sub")
t.expect_nothing_more()

t.cleanup()



#!/usr/bin/python

# Copyright (C) 2006. Vladimir Prus
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

import BoostBuild

def test_basic():
    t = BoostBuild.Tester(use_test_config=False)

    t.write("jamroot.jam", """\
exe a1 : a1.cpp : <conditional>@a1-rule ;
rule a1-rule ( properties * )
{
    if <variant>debug in $(properties)
    {
        return <define>OK ;
    }
}

exe a2 : a2.cpp : <conditional>@$(__name__).a2-rule
    <variant>debug:<optimization>speed ;
rule a2-rule ( properties * )
{
    if <optimization>speed in $(properties)
    {
        return <define>OK ;
    }
}

exe a3 : a3.cpp :
    <conditional>@$(__name__).a3-rule-1
    <conditional>@$(__name__).a3-rule-2 ;
rule a3-rule-1 ( properties * )
{
    if <optimization>speed in $(properties)
    {
        return <define>OK ;
    }
}
rule a3-rule-2 ( properties * )
{
    if <variant>debug in $(properties)
    {
        return <optimization>speed ;
    }
}
""")

    t.write("a1.cpp", "#ifdef OK\nint main() {}\n#endif\n")
    t.write("a2.cpp", "#ifdef OK\nint main() {}\n#endif\n")
    t.write("a3.cpp", "#ifdef OK\nint main() {}\n#endif\n")

    t.run_build_system()

    t.expect_addition("bin/$toolset/debug*/a1.exe")
    t.expect_addition("bin/$toolset/debug/optimization-speed*/a2.exe")
    t.expect_addition("bin/$toolset/debug/optimization-speed*/a3.exe")

    t.cleanup()


def test_glob_in_indirect_conditional():
    """
      Regression test: project-rules.glob rule run from inside an indirect
    conditional should report an error as it depends on the 'currently loaded
    project' concept and indirect conditional rules get called only after all
    the project modules have already finished loading.

    """
    t = BoostBuild.Tester(use_test_config=False)

    t.write("jamroot.jam", """\
use-project /library-example/foo : util/foo ;
build-project app ;
""")
    t.write("app/app.cpp", "int main() {}\n");
    t.write("app/jamfile.jam", "exe app : app.cpp /library-example/foo//bar ;")
    t.write("util/foo/bar.cpp", """\
#ifdef _WIN32
__declspec(dllexport)
#endif
void foo() {}
""")
    t.write("util/foo/jamfile.jam", """\
rule print-my-sources ( properties * )
{
    ECHO My sources: ;
    ECHO [ glob *.cpp ] ;
}
lib bar : bar.cpp : <conditional>@print-my-sources ;
""")

    t.run_build_system(status=1)
    t.expect_output_lines(["My sources:", "bar.cpp"], False)
    t.expect_output_lines("error: Reference to the project currently being "
        "loaded requested when there was no project module being loaded.")

    t.cleanup()


test_basic()
test_glob_in_indirect_conditional()



#!/usr/bin/python

# Copyright (C) Vladimir Prus 2003.
# Distributed under the Boost Software License, Version 1.0.
#    (See accompanying file LICENSE_1_0.txt or copy at
#         http://www.boost.org/LICENSE_1_0.txt)

# Test for the regression testing framework.

import BoostBuild

# Create a temporary working directory.
t = BoostBuild.Tester(use_test_config=False)

t.write("Jamroot.jam", """
project
  :
  : usage-requirements <define>TEST=1
  : default-build <link>static
;
message hello : "Hello World!" ;
alias hello : : <link>shared ;
obj test : test.cpp hello : <link>static ;
""")

t.write("test.cpp", """
#ifndef TEST
#error TEST not defined
#endif
""")

t.run_build_system(["test"], stdout="""Hello World!
""")

t.expect_addition("bin/$toolset/link-static*/test.obj")
t.expect_nothing_more()

t.cleanup()



# Status: ported.
# Base revision: 40480

#  Copyright (C) Vladimir Prus 2002. Permission to copy, use, modify, sell and
#  distribute this software is granted provided this copyright notice appears in
#  all copies. This software is provided "as is" without express or implied
#  warranty, and with no claim as to its suitability for any purpose.

import hashlib

import bjam
from b2.util.utility import *
import property, feature
import b2.build.feature
from b2.exceptions import *
from b2.build.property import get_abbreviated_paths
from b2.util.sequence import unique
from b2.util.set import difference
from b2.util import cached, abbreviate_dashed, is_iterable_typed

from b2.manager import get_manager


def reset ():
    """ Clear the module state. This is mainly for testing purposes.
    """
    global __cache

    # A cache of property sets
    # TODO: use a map of weak refs?
    __cache = {}

reset ()


def create (raw_properties = []):
    """ Creates a new 'PropertySet' instance for the given raw properties,
        or returns an already existing one.
    """
    assert (is_iterable_typed(raw_properties, property.Property)
            or is_iterable_typed(raw_properties, basestring))
    # FIXME: propagate to callers.
    if len(raw_properties) > 0 and isinstance(raw_properties[0], property.Property):
        x = raw_properties
    else:
        x = [property.create_from_string(ps) for ps in raw_properties]

    # These two lines of code are optimized to the current state
    # of the Property class. Since this function acts as the caching
    # frontend to the PropertySet class modifying these two lines
    # could have a severe performance penalty. Be careful.
    # It would be faster to sort by p.id, but some projects may rely
    # on the fact that the properties are ordered alphabetically. So,
    # we maintain alphabetical sorting so as to maintain backward compatibility.
    x = sorted(set(x), key=lambda p: (p.feature.name, p.value, p.condition))
    key = tuple(p.id for p in x)

    if key not in __cache:
        __cache [key] = PropertySet(x)

    return __cache [key]

def create_with_validation (raw_properties):
    """ Creates new 'PropertySet' instances after checking
        that all properties are valid and converting implicit
        properties into gristed form.
    """
    assert is_iterable_typed(raw_properties, basestring)
    properties = [property.create_from_string(s) for s in raw_properties]
    property.validate(properties)

    return create(properties)

def empty ():
    """ Returns PropertySet with empty set of properties.
    """
    return create ()

def create_from_user_input(raw_properties, jamfile_module, location):
    """Creates a property-set from the input given by the user, in the
    context of 'jamfile-module' at 'location'"""
    assert is_iterable_typed(raw_properties, basestring)
    assert isinstance(jamfile_module, basestring)
    assert isinstance(location, basestring)
    properties = property.create_from_strings(raw_properties, True)
    properties = property.translate_paths(properties, location)
    properties = property.translate_indirect(properties, jamfile_module)

    project_id = get_manager().projects().attributeDefault(jamfile_module, 'id', None)
    if not project_id:
        project_id = os.path.abspath(location)
    properties = property.translate_dependencies(properties, project_id, location)
    properties = property.expand_subfeatures_in_conditions(properties)
    return create(properties)


def refine_from_user_input(parent_requirements, specification, jamfile_module,
                           location):
    """Refines requirements with requirements provided by the user.
    Specially handles "-<property>value" syntax in specification
     to remove given requirements.
     - parent-requirements -- property-set object with requirements
       to refine
     - specification -- string list of requirements provided by the use
     - project-module -- the module to which context indirect features
       will be bound.
     - location -- the path to which path features are relative."""
    assert isinstance(parent_requirements, PropertySet)
    assert is_iterable_typed(specification, basestring)
    assert isinstance(jamfile_module, basestring)
    assert isinstance(location, basestring)

    if not specification:
        return parent_requirements


    add_requirements = []
    remove_requirements = []

    for r in specification:
        if r[0] == '-':
            remove_requirements.append(r[1:])
        else:
            add_requirements.append(r)

    if remove_requirements:
        # Need to create property set, so that path features
        # and indirect features are translated just like they
        # are in project requirements.
        ps = create_from_user_input(remove_requirements,
                                    jamfile_module, location)

        parent_requirements = create(difference(parent_requirements.all(),
                                                ps.all()))
        specification = add_requirements

    requirements = create_from_user_input(specification,
                                          jamfile_module, location)

    return parent_requirements.refine(requirements)

class PropertySet:
    """ Class for storing a set of properties.
        - there's 1<->1 correspondence between identity and value. No
          two instances of the class are equal. To maintain this property,
          the 'PropertySet.create' rule should be used to create new instances.
          Instances are immutable.

        - each property is classified with regard to it's effect on build
          results. Incidental properties have no effect on build results, from
          Boost.Build point of view. Others are either free, or non-free, which we
          call 'base'. Each property belong to exactly one of those categories and
          it's possible to get list of properties in each category.

          In addition, it's possible to get list of properties with specific
          attribute.

        - several operations, like and refine and as_path are provided. They all use
          caching whenever possible.
    """
    def __init__ (self, properties=None):
        if properties is None:
            properties = []
        assert is_iterable_typed(properties, property.Property)

        self.all_ = properties
        self._all_set = {p.id for p in properties}

        self.incidental_ = []
        self.free_ = []
        self.base_ = []
        self.dependency_ = []
        self.non_dependency_ = []
        self.conditional_ = []
        self.non_conditional_ = []
        self.propagated_ = []
        self.link_incompatible = []

        # A cache of refined properties.
        self.refined_ = {}

        # A cache of property sets created by adding properties to this one.
        self.added_ = {}

        # Cache for the default properties.
        self.defaults_ = None

        # Cache for the expanded properties.
        self.expanded_ = None

        # Cache for the expanded composite properties
        self.composites_ = None

        # Cache for property set with expanded subfeatures
        self.subfeatures_ = None

        # Cache for the property set containing propagated properties.
        self.propagated_ps_ = None

        # A map of features to its values.
        self.feature_map_ = None

        # A tuple (target path, is relative to build directory)
        self.target_path_ = None

        self.as_path_ = None

        # A cache for already evaluated sets.
        self.evaluated_ = {}

        # stores the list of LazyProperty instances.
        # these are being kept separate from the normal
        # Property instances so that when this PropertySet
        # tries to return one of its attributes, it
        # will then try to evaluate the LazyProperty instances
        # first before returning.
        self.lazy_properties = []

        for p in properties:
            f = p.feature
            if isinstance(p, property.LazyProperty):
                self.lazy_properties.append(p)
            # A feature can be both incidental and free,
            # in which case we add it to incidental.
            elif f.incidental:
                self.incidental_.append(p)
            elif f.free:
                self.free_.append(p)
            else:
                self.base_.append(p)

            if p.condition:
                self.conditional_.append(p)
            else:
                self.non_conditional_.append(p)

            if f.dependency:
                self.dependency_.append (p)
            elif not isinstance(p, property.LazyProperty):
                self.non_dependency_.append (p)

            if f.propagated:
                self.propagated_.append(p)
            if f.link_incompatible:
                self.link_incompatible.append(p)


    def all(self):
        return self.all_

    def raw (self):
        """ Returns the list of stored properties.
        """
        # create a new list due to the LazyProperties.
        # this gives them a chance to evaluate to their
        # true Property(). This approach is being
        # taken since calculations should not be using
        # PropertySet.raw()
        return [p._to_raw for p in self.all_]

    def __str__(self):
        return ' '.join(p._to_raw for p in self.all_)

    def base (self):
        """ Returns properties that are neither incidental nor free.
        """
        result = [p for p in self.lazy_properties
                  if not(p.feature.incidental or p.feature.free)]
        result.extend(self.base_)
        return result

    def free (self):
        """ Returns free properties which are not dependency properties.
        """
        result = [p for p in self.lazy_properties
                  if not p.feature.incidental and p.feature.free]
        result.extend(self.free_)
        return result

    def non_free(self):
        return self.base() + self.incidental()

    def dependency (self):
        """ Returns dependency properties.
        """
        result = [p for p in self.lazy_properties if p.feature.dependency]
        result.extend(self.dependency_)
        return self.dependency_

    def non_dependency (self):
        """ Returns properties that are not dependencies.
        """
        result = [p for p in self.lazy_properties if not p.feature.dependency]
        result.extend(self.non_dependency_)
        return result

    def conditional (self):
        """ Returns conditional properties.
        """
        return self.conditional_

    def non_conditional (self):
        """ Returns properties that are not conditional.
        """
        return self.non_conditional_

    def incidental (self):
        """ Returns incidental properties.
        """
        result = [p for p in self.lazy_properties if p.feature.incidental]
        result.extend(self.incidental_)
        return result

    def refine (self, requirements):
        """ Refines this set's properties using the requirements passed as an argument.
        """
        assert isinstance(requirements, PropertySet)
        if requirements not in self.refined_:
            r = property.refine(self.all_, requirements.all_)

            self.refined_[requirements] = create(r)

        return self.refined_[requirements]

    def expand (self):
        if not self.expanded_:
            expanded = feature.expand(self.all_)
            self.expanded_ = create(expanded)
        return self.expanded_

    def expand_subfeatures(self):
        if not self.subfeatures_:
            self.subfeatures_ = create(feature.expand_subfeatures(self.all_))
        return self.subfeatures_

    def evaluate_conditionals(self, context=None):
        assert isinstance(context, (PropertySet, type(None)))
        if not context:
            context = self

        if context not in self.evaluated_:
            # FIXME: figure why the call messes up first parameter
            self.evaluated_[context] = create(
                property.evaluate_conditionals_in_context(self.all(), context))

        return self.evaluated_[context]

    def propagated (self):
        if not self.propagated_ps_:
            self.propagated_ps_ = create (self.propagated_)
        return self.propagated_ps_

    def add_defaults (self):
        # FIXME: this caching is invalidated when new features
        # are declare inside non-root Jamfiles.
        if not self.defaults_:
            expanded = feature.add_defaults(self.all_)
            self.defaults_ = create(expanded)
        return self.defaults_

    def as_path (self):
        if not self.as_path_:

            def path_order (p1, p2):

                i1 = p1.feature.implicit
                i2 = p2.feature.implicit

                if i1 != i2:
                    return i2 - i1
                else:
                    return cmp(p1.feature.name, p2.feature.name)

            # trim redundancy
            properties = feature.minimize(self.base_)

            # sort according to path_order
            properties.sort (path_order)

            components = []
            for p in properties:
                f = p.feature
                if f.implicit:
                    components.append(p.value)
                else:
                    value = f.name.replace(':', '-') + "-" + p.value
                    if property.get_abbreviated_paths():
                        value = abbreviate_dashed(value)
                    components.append(value)

            self.as_path_ = '/'.join(components)

        return self.as_path_

    def target_path (self):
        """ Computes the target path that should be used for
            target with these properties.
            Returns a tuple of
              - the computed path
              - if the path is relative to build directory, a value of
                'true'.
        """
        if not self.target_path_:
            # The <location> feature can be used to explicitly
            # change the location of generated targets
            l = self.get ('<location>')
            if l:
                computed = l[0]
                is_relative = False

            else:
                p = self.as_path()
                if hash_maybe:
                    p = hash_maybe(p)

                # Really, an ugly hack. Boost regression test system requires
                # specific target paths, and it seems that changing it to handle
                # other directory layout is really hard. For that reason,
                # we teach V2 to do the things regression system requires.
                # The value o '<location-prefix>' is predended to the path.
                prefix = self.get ('<location-prefix>')

                if prefix:
                    if len (prefix) > 1:
                        raise AlreadyDefined ("Two <location-prefix> properties specified: '%s'" % prefix)

                    computed = os.path.join(prefix[0], p)

                else:
                    computed = p

                if not computed:
                    computed = "."

                is_relative = True

            self.target_path_ = (computed, is_relative)

        return self.target_path_

    def add (self, ps):
        """ Creates a new property set containing the properties in this one,
            plus the ones of the property set passed as argument.
        """
        assert isinstance(ps, PropertySet)
        if ps not in self.added_:
            self.added_[ps] = create(self.all_ + ps.all())
        return self.added_[ps]

    def add_raw (self, properties):
        """ Creates a new property set containing the properties in this one,
            plus the ones passed as argument.
        """
        return self.add (create (properties))


    def get (self, feature):
        """ Returns all values of 'feature'.
        """
        if type(feature) == type([]):
            feature = feature[0]
        if not isinstance(feature, b2.build.feature.Feature):
            feature = b2.build.feature.get(feature)
        assert isinstance(feature, b2.build.feature.Feature)

        if self.feature_map_ is None:
            self.feature_map_ = {}

            for v in self.all_:
                if v.feature not in self.feature_map_:
                    self.feature_map_[v.feature] = []
                self.feature_map_[v.feature].append(v.value)

        return self.feature_map_.get(feature, [])

    @cached
    def get_properties(self, feature):
        """Returns all contained properties associated with 'feature'"""
        if not isinstance(feature, b2.build.feature.Feature):
            feature = b2.build.feature.get(feature)
        assert isinstance(feature, b2.build.feature.Feature)

        result = []
        for p in self.all_:
            if p.feature == feature:
                result.append(p)
        return result

    def __contains__(self, item):
        return item.id in self._all_set

def hash(p):
    m = hashlib.md5()
    m.update(p)
    return m.hexdigest()

hash_maybe = hash if "--hash" in bjam.variable("ARGV") else None




#!/usr/bin/python

# Copyright (C) Vladimir Prus 2010.
# Distributed under the Boost Software License, Version 1.0. (See
# accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

# Test that build failure results in non-zero exit status

import BoostBuild

# Create a temporary working directory.
t = BoostBuild.Tester()

# Create the needed files.
t.write("jamroot.jam", """
exe hello : hello.cpp ;
""")

t.write("hello.cpp", """
int main() {
""")

t.run_build_system(status=1)

t.cleanup()



#!/usr/bin/python

# Copyright 2007 Rene Rivera.
# Copyright 2011 Steven Watanabe
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

import BoostBuild

t = BoostBuild.Tester(pass_toolset=0, pass_d0=False)

t.write("file.jam", """\
actions .a.
{
echo [$(<:B)] 0
echo [$(<:B)] 1
echo [$(<:B)] 2
}

rule .a.
{
    DEPENDS $(<) : $(>) ;
}

NOTFILE subtest ;
.a. subtest_a : subtest ;
.a. subtest_b : subtest ;
FAIL_EXPECTED subtest_b ;
DEPENDS all : subtest_a subtest_b ;
""")

t.run_build_system(["-ffile.jam", "-n"], stdout="""\
...found 4 targets...
...updating 2 targets...
.a. subtest_a

echo [subtest_a] 0
echo [subtest_a] 1
echo [subtest_a] 2

.a. subtest_b

echo [subtest_b] 0
echo [subtest_b] 1
echo [subtest_b] 2

...updated 2 targets...
""")
t.expect_nothing_more()

t.cleanup()



#!/usr/bin/python

# Copyright (C) 2013 Steven Watanabe
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

import sys

def create(t):
  t.write('''mockinfo.py''', '''
import re
import optparse
import os

parser = optparse.OptionParser()
parser.add_option('-o', dest="output_file")
parser.add_option('-x', dest="language")
parser.add_option('-c', dest="compile", action="store_true")
parser.add_option('-I', dest="includes", action="append")
parser.add_option('-L', dest="library_path", action="append")
parser.add_option('--dll', dest="dll", action="store_true")
parser.add_option('--archive', dest="archive", action="store_true")
parser.add_option('--static-lib', dest="static_libraries", action="append")
parser.add_option('--shared-lib', dest="shared_libraries", action="append")

cwd = os.environ["JAM_CWD"]

class MockInfo(object):
  def __init__(self, verbose=False):
    self.files = dict()
    self.commands = list()
    self.verbose = verbose
  def source_file(self, name, pattern):
    self.files[name] = pattern
  def action(self, command, status=0):
    self.commands.append((command, status))
  def check(self, command):
    print "Testing command", command
    for (raw, status) in self.commands:
      if self.matches(raw, command):
        return status
  def matches(self, raw, command):
    (expected_options, expected_args) = parser.parse_args(raw.split())
    options = command[0]
    input_files = list(command[1])
    if self.verbose:
      print "  - matching against", (expected_options, expected_args)
    if len(expected_args) != len(input_files):
      if self.verbose:
        print "  argument list sizes differ"
      return False
    for arg in expected_args:
      if arg.startswith('$'):
        fileid = arg[1:]
        pattern = self.files[fileid] if fileid in self.files else fileid
        matching_file = None
        for input_file in input_files:
          with open(input_file, 'r') as f:
            contents = f.read()
          if pattern == contents:
            matching_file = input_file
            break
        if matching_file is not None:
          input_files.remove(matching_file)
        else:
          if self.verbose:
            print "    Failed to match input file contents: %s" % arg
          return False
      else:
        if arg in input_files:
          input_files.remove(arg)
        else:
          if self.verbose:
            print "    Failed to match input file: %s" % arg
          return False

    if options.language != expected_options.language:
      if self.verbose:
        print "    Failed to match -c"
      return False

    if options.compile != expected_options.compile:
      if self.verbose:
        print "    Failed to match -x"
      return False

    # Normalize a path for comparison purposes
    def adjust_path(p):
      return os.path.normcase(os.path.normpath(os.path.join(cwd, p)))

    # order matters
    if options.includes is None:
      options.includes = []
    if expected_options.includes is None:
      expected_options.includes = []
    if map(adjust_path, options.includes) != \
        map(adjust_path, expected_options.includes):
      if self.verbose:
        print "    Failed to match -I ",  map(adjust_path, options.includes), \
          " != ", map(adjust_path, expected_options.includes)
      return False

    if options.library_path is None:
      options.library_path = []
    if expected_options.library_path is None:
      expected_options.library_path = []
    if map(adjust_path, options.library_path) != \
        map(adjust_path, expected_options.library_path):
      if self.verbose:
        print "    Failed to match -L ",  map(adjust_path, options.library_path), \
          " != ", map(adjust_path, expected_options.library_path)
      return False

    if options.static_libraries != expected_options.static_libraries:
      if self.verbose:
        print "    Failed to match --static-lib"
      return False

    if options.shared_libraries != expected_options.shared_libraries:
      if self.verbose:
        print "    Failed to match --shared-lib"
      return False

    if options.dll != expected_options.dll:
      if self.verbose:
        print "    Failed to match --dll"
      return False

    if options.archive != expected_options.archive:
      if self.verbose:
        print "    Failed to match --archive"
      return False

    # The output must be handled after everything else
    # is validated
    if expected_options.output_file is not None:
      if options.output_file is not None:
        if expected_options.output_file.startswith('$'):
          fileid = expected_options.output_file[1:]
          if fileid not in self.files:
            self.files[fileid] = fileid
          else:
            assert(self.files[fileid] == fileid)
          with open(options.output_file, 'w') as output:
            output.write(fileid)
      else:
        if self.verbose:
          print "Failed to match -o"
        return False
    elif options.output_file is not None:
      if self.verbose:
        print "Failed to match -o"
      return False

    # if we've gotten here, then everything matched
    if self.verbose:
      print "    Matched"
    return True
''')

  t.write('mock.py', '''
import mockinfo
import markup
import sys

status = markup.info.check(mockinfo.parser.parse_args())
if status is not None:
  exit(status)
else:
  print("Unrecognized command: " + ' '.join(sys.argv))
  exit(1)
''')

  t.write('mock.jam', '''
import feature ;
import toolset ;
import path ;
import modules ;
import common ;
import type ;

.python-cmd = "\"%s\"" ;

# Behave the same as gcc on Windows, because that's what
# the test system expects
type.set-generated-target-prefix SHARED_LIB : <toolset>mock <target-os>windows : lib ;
type.set-generated-target-suffix STATIC_LIB : <toolset>mock <target-os>windows : a ;

rule init ( )
{
    local here = [ path.make [ modules.binding $(__name__) ] ] ;
    here = [ path.native [ path.root [ path.parent $(here) ] [ path.pwd ] ] ] ;
    .config-cmd = [ common.variable-setting-command JAM_CWD : $(here) ] $(.python-cmd) -B ;
}

feature.extend toolset : mock ;

generators.register-c-compiler mock.compile.c++ : CPP : OBJ : <toolset>mock ;
generators.register-c-compiler mock.compile.c : C : OBJ : <toolset>mock ;

generators.register-linker mock.link : LIB OBJ : EXE : <toolset>mock ;
generators.register-linker mock.link.dll : LIB OBJ : SHARED_LIB : <toolset>mock ;
generators.register-archiver mock.archive : OBJ : STATIC_LIB : <toolset>mock ;

toolset.flags mock.compile INCLUDES <include> ;

actions compile.c
{
   $(.config-cmd) mock.py -c -x c -I"$(INCLUDES)" "$(>)" -o "$(<)"
}

actions compile.c++
{
    $(.config-cmd) mock.py -c -x c++ -I"$(INCLUDES)" "$(>)" -o "$(<)"
}

toolset.flags mock.link USER_OPTIONS <linkflags> ;
toolset.flags mock.link FINDLIBS-STATIC <find-static-library> ;
toolset.flags mock.link FINDLIBS-SHARED <find-shared-library> ;
toolset.flags mock.link LINK_PATH <library-path> ;
toolset.flags mock.link LIBRARIES <library-file> ;

actions link
{
    $(.config-cmd) mock.py "$(>)" -o "$(<)" $(USER_OPTIONS) -L"$(LINK_PATH)" --static-lib=$(FINDLIBS-STATIC) --shared-lib=$(FINDLIBS-SHARED)
}

actions archive
{
    $(.config-cmd) mock.py --archive "$(>)" -o "$(<)" $(USER_OPTIONS)
}

actions link.dll
{
    $(.config-cmd) mock.py --dll "$(>)" -o "$(<)" $(USER_OPTIONS) -L"$(LINK_PATH)" --static-lib=$(FINDLIBS-STATIC) --shared-lib=$(FINDLIBS-SHARED)
}

''' % sys.executable.replace('\\', '\\\\'))

def set_expected(t, markup):
  verbose = "True" if t.verbose else "False"
  t.write('markup.py', '''
import mockinfo
info = mockinfo.MockInfo(%s)
def source_file(name, contents):
  info.source_file(name, contents)
def action(command, status=0):
  info.action(command, status)
''' % verbose + markup)



#!/usr/bin/python

# Copyright (C) 2006. Vladimir Prus
# Copyright (C) 2008. Jurko Gospodnetic
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or copy at
# http://www.boost.org/LICENSE_1_0.txt)

# Tests that we explicitly request a file (not target) to be built by
# specifying its name on the command line.

import BoostBuild


###############################################################################
#
# test_building_file_from_specific_project()
# ------------------------------------------
#
###############################################################################

def test_building_file_from_specific_project():
    t = BoostBuild.Tester(use_test_config=False)

    t.write("jamroot.jam", """\
exe hello : hello.cpp ;
exe hello2 : hello.cpp ;
build-project sub ;
""")
    t.write("hello.cpp", "int main() {}\n")
    t.write("sub/jamfile.jam", """
exe hello : hello.cpp ;
exe hello2 : hello.cpp ;
exe sub : hello.cpp ;
""")
    t.write("sub/hello.cpp", "int main() {}\n")

    t.run_build_system(["sub", t.adjust_suffix("hello.obj")])
    t.expect_output_lines("*depends on itself*", False)
    t.expect_addition("sub/bin/$toolset/debug*/hello.obj")
    t.expect_nothing_more()

    t.cleanup()


###############################################################################
#
# test_building_file_from_specific_target()
# -----------------------------------------
#
###############################################################################

def test_building_file_from_specific_target():
    t = BoostBuild.Tester(use_test_config=False)

    t.write("jamroot.jam", """\
exe hello1 : hello1.cpp ;
exe hello2 : hello2.cpp ;
exe hello3 : hello3.cpp ;
""")
    t.write("hello1.cpp", "int main() {}\n")
    t.write("hello2.cpp", "int main() {}\n")
    t.write("hello3.cpp", "int main() {}\n")

    t.run_build_system(["hello1", t.adjust_suffix("hello1.obj")])
    t.expect_addition("bin/$toolset/debug*/hello1.obj")
    t.expect_nothing_more()

    t.cleanup()


###############################################################################
#
# test_building_missing_file_from_specific_target()
# -------------------------------------------------
#
###############################################################################

def test_building_missing_file_from_specific_target():
    t = BoostBuild.Tester(use_test_config=False)

    t.write("jamroot.jam", """\
exe hello1 : hello1.cpp ;
exe hello2 : hello2.cpp ;
exe hello3 : hello3.cpp ;
""")
    t.write("hello1.cpp", "int main() {}\n")
    t.write("hello2.cpp", "int main() {}\n")
    t.write("hello3.cpp", "int main() {}\n")

    obj = t.adjust_suffix("hello2.obj")
    t.run_build_system(["hello1", obj], status=1)
    t.expect_output_lines("don't know how to make*" + obj)
    t.expect_nothing_more()

    t.cleanup()


###############################################################################
#
# test_building_multiple_files_with_different_names()
# ---------------------------------------------------
#
###############################################################################

def test_building_multiple_files_with_different_names():
    t = BoostBuild.Tester(use_test_config=False)

    t.write("jamroot.jam", """\
exe hello1 : hello1.cpp ;
exe hello2 : hello2.cpp ;
exe hello3 : hello3.cpp ;
""")
    t.write("hello1.cpp", "int main() {}\n")
    t.write("hello2.cpp", "int main() {}\n")
    t.write("hello3.cpp", "int main() {}\n")

    t.run_build_system([t.adjust_suffix("hello1.obj"), t.adjust_suffix(
        "hello2.obj")])
    t.expect_addition("bin/$toolset/debug*/hello1.obj")
    t.expect_addition("bin/$toolset/debug*/hello2.obj")
    t.expect_nothing_more()

    t.cleanup()


###############################################################################
#
# test_building_multiple_files_with_the_same_name()
# -------------------------------------------------
#
###############################################################################

def test_building_multiple_files_with_the_same_name():
    t = BoostBuild.Tester(use_test_config=False)

    t.write("jamroot.jam", """\
exe hello : hello.cpp ;
exe hello2 : hello.cpp ;
build-project sub ;
""")
    t.write("hello.cpp", "int main() {}\n")
    t.write("sub/jamfile.jam", """
exe hello : hello.cpp ;
exe hello2 : hello.cpp ;
exe sub : hello.cpp ;
""")
    t.write("sub/hello.cpp", "int main() {}\n")

    t.run_build_system([t.adjust_suffix("hello.obj")])
    t.expect_output_lines("*depends on itself*", False)
    t.expect_addition("bin/$toolset/debug*/hello.obj")
    t.expect_addition("sub/bin/$toolset/debug*/hello.obj")
    t.expect_nothing_more()

    t.cleanup()


###############################################################################
#
# main()
# ------
#
###############################################################################

test_building_file_from_specific_project()
test_building_file_from_specific_target()
test_building_missing_file_from_specific_target()
test_building_multiple_files_with_different_names()
test_building_multiple_files_with_the_same_name()



#!/usr/bin/python

# Copyright 2012 Jurko Gospodnetic
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

#   Tests for a bug causing Boost Build's scanner targets to be rebuilt.
# unnecessarily in the following scenario:
#  * We want to build target X requiring target A.
#  * We have a multi-file action generating targets A & B.
#  * Out action generates target B with a more recent timestamp than target A.
#  * Target A includes target B.
#  * Target A has a registered include scanner.
# Now even if our targets A & B have already been built and are up-to-date
# (e.g. in a state left by a previous successful build run), our scanner target
# tasked with scanning target A will be marked for updating, thus causing any
# targets depending on it to be updated/rebuilt as well.

import BoostBuild

t = BoostBuild.Tester(use_test_config=False)

t.write("foo.jam", r"""
import common ;
import generators ;
import modules ;
import type ;
import types/cpp ;

type.register FOO : foo ;
type.register BAR : bar ;
generators.register-standard foo.foo : FOO : CPP BAR ;

local rule sleep-cmd ( delay )
{
    if [ modules.peek : NT ]
    {
        return ping 127.0.0.1 -n $(delay) -w 1000 >NUL ;
    }
    else
    {
        return sleep $(delay) ;
    }
}

.touch = [ common.file-creation-command ] ;
.sleep = [ sleep-cmd 2 ] ;

rule foo ( cpp bar : foo : properties * )
{
    # We add the INCLUDE relationship between our generated CPP & BAR targets
    # explicitly instead of relying on Boost Jam's internal implementation
    # detail - automatically adding such relationships between all files
    # generated by the same action. This way our test will continue to function
    # correctly even if the related Boost Jam implementation detail changes.
    # Note that adding this relationship by adding an #include directive in our
    # generated CPP file is not good enough as such a relationship would get
    # added only after the scanner target's relationships have already been
    # established and they (as affected by our initial INCLUDE relationship) are
    # the original reason for this test failing.
    INCLUDES $(cpp) : $(bar) ;
}

actions foo
{
    $(.touch) "$(<[1])"
    $(.sleep)
    $(.touch) "$(<[2])"
}
""")

t.write(
    'foo.py',
"""
import os

from b2.build import type as type_, generators
from b2.tools import common
from b2.manager import get_manager

MANAGER = get_manager()
ENGINE = MANAGER.engine()

type_.register('FOO', ['foo'])
type_.register('BAR', ['bar'])
generators.register_standard('foo.foo', ['FOO'], ['CPP', 'BAR'])

def sleep_cmd(delay):
    if os.name == 'nt':
        return 'ping 127.0.0.1 -n {} -w 1000 >NUL'.format(delay)
    return 'sleep {}'.format(delay)

def foo(targets, sources, properties):
    cpp, bar = targets
    foo = sources[0]
    # We add the INCLUDE relationship between our generated CPP & BAR targets
    # explicitly instead of relying on Boost Jam's internal implementation
    # detail - automatically adding such relationships between all files
    # generated by the same action. This way our test will continue to function
    # correctly even if the related Boost Jam implementation detail changes.
    # Note that adding this relationship by adding an #include directive in our
    # generated CPP file is not good enough as such a relationship would get
    # added only after the scanner target's relationships have already been
    # established and they (as affected by our initial INCLUDE relationship) are
    # the original reason for this test failing.
    bjam.call('INCLUDES', cpp, bar)

ENGINE.register_action(
    'foo.foo',
    '''
    {touch} "$(<[1])"
    {sleep}
    {touch} "$(<[2])"
    '''.format(touch=common.file_creation_command(), sleep=sleep_cmd(2))
)
"""
)

t.write("x.foo", "")
t.write("jamroot.jam", """\
import foo ;
lib x : x.foo : <link>static ;
""")


# Get everything built once.
t.run_build_system()

# Simply rerunning the build without touching any of its source target files
# should not cause any files to be affected.
t.run_build_system()
t.expect_nothing_more()



#!/usr/bin/python

# Copyright 2002, 2003 Dave Abrahams
# Copyright 2002, 2003 Vladimir Prus
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

import BoostBuild

t = BoostBuild.Tester()

t.set_tree("test2")

file_list = 'bin/$toolset/debug*/' * \
    BoostBuild.List("foo foo.o")

t.run_build_system("-sBOOST_BUILD_PATH=" + t.original_workdir + "/..")
t.expect_addition(file_list)


t.write("foo.cpp", "int main() {}\n")
t.run_build_system("-d2 -sBOOST_BUILD_PATH=" + t.original_workdir + "/..")
t.expect_touch(file_list)

t.cleanup()



#!/usr/bin/python

# Copyright 2002 Vladimir Prus
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

# Test that we cannot specify targets of unknown type as sources. This is based
# on the fact that Unix 'ar' will happily consume just about anything.

import BoostBuild

t = BoostBuild.Tester()

t.write("jamroot.jam", "static-lib a : a.foo ;")
t.write("a.foo", "")

t.run_build_system(status=1)

t.cleanup()



#!/usr/bin/python

# Copyright 2003, 2004 Vladimir Prus
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

# Test that sources with absolute names are handled OK.

import BoostBuild

t = BoostBuild.Tester(use_test_config=False)

t.write("jamroot.jam", "path-constant TOP : . ;")
t.write("jamfile.jam", """\
local pwd = [ PWD ] ;
ECHO $(pwd) XXXXX ;
exe hello : $(pwd)/hello.cpp $(TOP)/empty.cpp ;
""")
t.write("hello.cpp", "int main() {}\n")
t.write("empty.cpp", "\n")

t.run_build_system()
t.expect_addition("bin/$toolset/debug*/hello.exe")
t.rm(".")

# Test a contrived case in which an absolute name is used in a standalone
# project (not Jamfile). Moreover, the target with an absolute name is returned
# via an 'alias' and used from another project.
t.write("a.cpp", "int main() {}\n")
t.write("jamfile.jam", "exe a : /standalone//a ;")
t.write("jamroot.jam", "import standalone ;")
t.write("standalone.jam", """\
import project ;
project.initialize $(__name__) ;
project standalone ;
local pwd = [ PWD ] ;
alias a : $(pwd)/a.cpp ;
""")

t.write("standalone.py", """
from b2.manager import get_manager

# FIXME: this is ugly as death
get_manager().projects().initialize(__name__)

import os ;

# This use of list as parameter is also ugly.
project(['standalone'])

pwd = os.getcwd()
alias('a', [os.path.join(pwd, 'a.cpp')])
""")

t.run_build_system()
t.expect_addition("bin/$toolset/debug*/a.exe")

# Test absolute path in target ids.
t.rm(".")

t.write("d1/jamroot.jam", "")
t.write("d1/jamfile.jam", "exe a : a.cpp ;")
t.write("d1/a.cpp", "int main() {}\n")
t.write("d2/jamroot.jam", "")
t.write("d2/jamfile.jam", """\
local pwd = [ PWD ] ;
alias x : $(pwd)/../d1//a ;
""")

t.run_build_system(subdir="d2")
t.expect_addition("d1/bin/$toolset/debug*/a.exe")

t.cleanup()



#!/usr/bin/python

# Copyright 2012 Steven Watanabe
# Distributed under the Boost Software License, Version 1.0.
# (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)

import BoostBuild

t = BoostBuild.Tester(use_test_config=False)

# Test a header loop that depends on (but does not contain) a generated header.
t.write("test.cpp", '#include "header1.h"\n')

t.write("header1.h", """\
#ifndef HEADER1_H
#define HEADER1_H
#include "header2.h"
#endif
""")

t.write("header2.h", """\
#ifndef HEADER2_H
#define HEADER2_H
#include "header1.h"
#include "header3.h"
#endif
""")

t.write("header3.in", "/* empty file */\n")

t.write("jamroot.jam", """\
import common ;
make header3.h : header3.in : @common.copy ;
obj test : test.cpp : <implicit-dependency>header3.h ;
""")

t.run_build_system(["-j2"])
t.expect_addition("bin/$toolset/debug*/header3.h")
t.expect_addition("bin/$toolset/debug*/test.obj")
t.expect_nothing_more()

t.rm(".")

# Test a linear sequence of generated headers.
t.write("test.cpp", '#include "header1.h"\n')

t.write("header1.in", """\
#ifndef HEADER1_H
#define HEADER1_H
#include "header2.h"
#endif
""")

t.write("header2.in", """\
#ifndef HEADER2_H
#define HEADER2_H
#include "header3.h"
#endif
""")

t.write("header3.in", "/* empty file */\n")

t.write("jamroot.jam", """\
import common ;
make header1.h : header1.in : @common.copy ;
make header2.h : header2.in : @common.copy ;
make header3.h : header3.in : @common.copy ;
obj test : test.cpp :
    <implicit-dependency>header1.h
    <implicit-dependency>header2.h
    <implicit-dependency>header3.h ;
""")

t.run_build_system(["-j2", "test"])
t.expect_addition("bin/$toolset/debug*/header1.h")
t.expect_addition("bin/$toolset/debug*/header2.h")
t.expect_addition("bin/$toolset/debug*/header3.h")
t.expect_addition("bin/$toolset/debug*/test.obj")
t.expect_nothing_more()

t.rm(".")

# Test a loop in generated headers.
t.write("test.cpp", '#include "header1.h"\n')

t.write("header1.in", """\
#ifndef HEADER1_H
#define HEADER1_H
#include "header2.h"
#endif
""")

t.write("header2.in", """\
#ifndef HEADER2_H
#define HEADER2_H
#include "header3.h"
#endif
""")

t.write("header3.in", """\
#ifndef HEADER2_H
#define HEADER2_H
#include "header1.h"
#endif
""")

t.write("jamroot.jam", """\
import common ;

actions copy {
    sleep 1
    cp $(>) $(<)
}

make header1.h : header1.in : @common.copy ;
make header2.h : header2.in : @common.copy ;
make header3.h : header3.in : @common.copy ;
obj test : test.cpp :
    <implicit-dependency>header1.h
    <implicit-dependency>header2.h
    <implicit-dependency>header3.h ;
""")

t.run_build_system(["-j2", "test"])
t.expect_addition("bin/$toolset/debug*/header1.h")
t.expect_addition("bin/$toolset/debug*/header2.h")
t.expect_addition("bin/$toolset/debug*/header3.h")
t.expect_addition("bin/$toolset/debug*/test.obj")
t.expect_nothing_more()

t.rm(".")

# Test that all the dependencies of a loop are updated before any of the
# dependents.
t.write("test1.cpp", '#include "header1.h"\n')

t.write("test2.cpp", """\
#include "header2.h"
int main() {}
""")

t.write("header1.h", """\
#ifndef HEADER1_H
#define HEADER1_H
#include "header2.h"
#endif
""")

t.write("header2.h", """\
#ifndef HEADER2_H
#define HEADER2_H
#include "header1.h"
#include "header3.h"
#endif
""")

t.write("header3.in", "\n")

t.write("sleep.bat", """\
::@timeout /T %1 /NOBREAK >nul
@ping 127.0.0.1 -n 2 -w 1000 >nul
@ping 127.0.0.1 -n %1 -w 1000 >nul
@exit /B 0
""")

t.write("jamroot.jam", """\
import common ;
import os ;

if [ os.name ] = NT
{
    SLEEP = call sleep.bat ;
}
else
{
    SLEEP = sleep ;
}

rule copy { common.copy $(<) : $(>) ; }
actions copy { $(SLEEP) 1 }

make header3.h : header3.in : @copy ;
exe test : test2.cpp test1.cpp : <implicit-dependency>header3.h ;
""")

t.run_build_system(["-j2", "test"])
t.expect_addition("bin/$toolset/debug*/header3.h")
t.expect_addition("bin/$toolset/debug*/test1.obj")
t.expect_addition("bin/$toolset/debug*/test2.obj")
t.expect_addition("bin/$toolset/debug*/test.exe")
t.expect_nothing_more()

t.touch("header3.in")
t.run_build_system(["-j2", "test"])
t.expect_touch("bin/$toolset/debug*/header3.h")
t.expect_touch("bin/$toolset/debug*/test1.obj")
t.expect_touch("bin/$toolset/debug*/test2.obj")
t.expect_touch("bin/$toolset/debug*/test.exe")
t.expect_nothing_more()

t.rm(".")

# Test a loop that includes a generated header
t.write("test1.cpp", '#include "header1.h"\n')
t.write("test2.cpp", """\
#include "header2.h"
int main() {}
""")

t.write("header1.h", """\
#ifndef HEADER1_H
#define HEADER1_H
#include "header2.h"
#endif
""")

t.write("header2.in", """\
#ifndef HEADER2_H
#define HEADER2_H
#include "header3.h"
#endif
""")

t.write("header3.h", """\
#ifndef HEADER3_H
#define HEADER3_H
#include "header1.h"
#endif
""")

t.write("sleep.bat", """\
::@timeout /T %1 /NOBREAK >nul
@ping 127.0.0.1 -n 2 -w 1000 >nul
@ping 127.0.0.1 -n %1 -w 1000 >nul
@exit /B 0
""")

t.write("jamroot.jam", """\
import common ;
import os ;

if [ os.name ] = NT
{
    SLEEP = call sleep.bat ;
}
else
{
    SLEEP = sleep ;
}

rule copy { common.copy $(<) : $(>) ; }
actions copy { $(SLEEP) 1 }

make header2.h : header2.in : @copy ;
exe test : test2.cpp test1.cpp : <implicit-dependency>header2.h <include>. ;
""")

t.run_build_system(["-j2", "test"])
t.expect_addition("bin/$toolset/debug*/header2.h")
t.expect_addition("bin/$toolset/debug*/test1.obj")
t.expect_addition("bin/$toolset/debug*/test2.obj")
t.expect_addition("bin/$toolset/debug*/test.exe")
t.expect_nothing_more()

t.cleanup()
